#line 1 "grecs-lex.c"
#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#line 6 "grecs-lex.c"

#define  YY_INT_ALIGNED short int

/* A lexical scanner generated by flex */

/* %not-for-header */
/* %if-c-only */
/* %if-not-reentrant */
/* %endif */
/* %endif */
/* %ok-for-header */

#define FLEX_SCANNER
#define YY_FLEX_MAJOR_VERSION 2
#define YY_FLEX_MINOR_VERSION 6
#define YY_FLEX_SUBMINOR_VERSION 4
#if YY_FLEX_SUBMINOR_VERSION > 0
#define FLEX_BETA
#endif

/* %if-c++-only */
/* %endif */

/* %if-c-only */
#ifdef yy_create_buffer
#define grecs_grecs__create_buffer_ALREADY_DEFINED
#else
#define yy_create_buffer grecs_grecs__create_buffer
#endif

#ifdef yy_delete_buffer
#define grecs_grecs__delete_buffer_ALREADY_DEFINED
#else
#define yy_delete_buffer grecs_grecs__delete_buffer
#endif

#ifdef yy_scan_buffer
#define grecs_grecs__scan_buffer_ALREADY_DEFINED
#else
#define yy_scan_buffer grecs_grecs__scan_buffer
#endif

#ifdef yy_scan_string
#define grecs_grecs__scan_string_ALREADY_DEFINED
#else
#define yy_scan_string grecs_grecs__scan_string
#endif

#ifdef yy_scan_bytes
#define grecs_grecs__scan_bytes_ALREADY_DEFINED
#else
#define yy_scan_bytes grecs_grecs__scan_bytes
#endif

#ifdef yy_init_buffer
#define grecs_grecs__init_buffer_ALREADY_DEFINED
#else
#define yy_init_buffer grecs_grecs__init_buffer
#endif

#ifdef yy_flush_buffer
#define grecs_grecs__flush_buffer_ALREADY_DEFINED
#else
#define yy_flush_buffer grecs_grecs__flush_buffer
#endif

#ifdef yy_load_buffer_state
#define grecs_grecs__load_buffer_state_ALREADY_DEFINED
#else
#define yy_load_buffer_state grecs_grecs__load_buffer_state
#endif

#ifdef yy_switch_to_buffer
#define grecs_grecs__switch_to_buffer_ALREADY_DEFINED
#else
#define yy_switch_to_buffer grecs_grecs__switch_to_buffer
#endif

#ifdef yypush_buffer_state
#define grecs_grecs_push_buffer_state_ALREADY_DEFINED
#else
#define yypush_buffer_state grecs_grecs_push_buffer_state
#endif

#ifdef yypop_buffer_state
#define grecs_grecs_pop_buffer_state_ALREADY_DEFINED
#else
#define yypop_buffer_state grecs_grecs_pop_buffer_state
#endif

#ifdef yyensure_buffer_stack
#define grecs_grecs_ensure_buffer_stack_ALREADY_DEFINED
#else
#define yyensure_buffer_stack grecs_grecs_ensure_buffer_stack
#endif

#ifdef yylex
#define grecs_grecs_lex_ALREADY_DEFINED
#else
#define yylex grecs_grecs_lex
#endif

#ifdef yyrestart
#define grecs_grecs_restart_ALREADY_DEFINED
#else
#define yyrestart grecs_grecs_restart
#endif

#ifdef yylex_init
#define grecs_grecs_lex_init_ALREADY_DEFINED
#else
#define yylex_init grecs_grecs_lex_init
#endif

#ifdef yylex_init_extra
#define grecs_grecs_lex_init_extra_ALREADY_DEFINED
#else
#define yylex_init_extra grecs_grecs_lex_init_extra
#endif

#ifdef yylex_destroy
#define grecs_grecs_lex_destroy_ALREADY_DEFINED
#else
#define yylex_destroy grecs_grecs_lex_destroy
#endif

#ifdef yyget_debug
#define grecs_grecs_get_debug_ALREADY_DEFINED
#else
#define yyget_debug grecs_grecs_get_debug
#endif

#ifdef yyset_debug
#define grecs_grecs_set_debug_ALREADY_DEFINED
#else
#define yyset_debug grecs_grecs_set_debug
#endif

#ifdef yyget_extra
#define grecs_grecs_get_extra_ALREADY_DEFINED
#else
#define yyget_extra grecs_grecs_get_extra
#endif

#ifdef yyset_extra
#define grecs_grecs_set_extra_ALREADY_DEFINED
#else
#define yyset_extra grecs_grecs_set_extra
#endif

#ifdef yyget_in
#define grecs_grecs_get_in_ALREADY_DEFINED
#else
#define yyget_in grecs_grecs_get_in
#endif

#ifdef yyset_in
#define grecs_grecs_set_in_ALREADY_DEFINED
#else
#define yyset_in grecs_grecs_set_in
#endif

#ifdef yyget_out
#define grecs_grecs_get_out_ALREADY_DEFINED
#else
#define yyget_out grecs_grecs_get_out
#endif

#ifdef yyset_out
#define grecs_grecs_set_out_ALREADY_DEFINED
#else
#define yyset_out grecs_grecs_set_out
#endif

#ifdef yyget_leng
#define grecs_grecs_get_leng_ALREADY_DEFINED
#else
#define yyget_leng grecs_grecs_get_leng
#endif

#ifdef yyget_text
#define grecs_grecs_get_text_ALREADY_DEFINED
#else
#define yyget_text grecs_grecs_get_text
#endif

#ifdef yyget_lineno
#define grecs_grecs_get_lineno_ALREADY_DEFINED
#else
#define yyget_lineno grecs_grecs_get_lineno
#endif

#ifdef yyset_lineno
#define grecs_grecs_set_lineno_ALREADY_DEFINED
#else
#define yyset_lineno grecs_grecs_set_lineno
#endif

#ifdef yyget_column
#define grecs_grecs_get_column_ALREADY_DEFINED
#else
#define yyget_column grecs_grecs_get_column
#endif

#ifdef yyset_column
#define grecs_grecs_set_column_ALREADY_DEFINED
#else
#define yyset_column grecs_grecs_set_column
#endif

#ifdef yywrap
#define grecs_grecs_wrap_ALREADY_DEFINED
#else
#define yywrap grecs_grecs_wrap
#endif

/* %endif */

#ifdef yyget_lval
#define grecs_grecs_get_lval_ALREADY_DEFINED
#else
#define yyget_lval grecs_grecs_get_lval
#endif

#ifdef yyset_lval
#define grecs_grecs_set_lval_ALREADY_DEFINED
#else
#define yyset_lval grecs_grecs_set_lval
#endif

#ifdef yyget_lloc
#define grecs_grecs_get_lloc_ALREADY_DEFINED
#else
#define yyget_lloc grecs_grecs_get_lloc
#endif

#ifdef yyset_lloc
#define grecs_grecs_set_lloc_ALREADY_DEFINED
#else
#define yyset_lloc grecs_grecs_set_lloc
#endif

#ifdef yyalloc
#define grecs_grecs_alloc_ALREADY_DEFINED
#else
#define yyalloc grecs_grecs_alloc
#endif

#ifdef yyrealloc
#define grecs_grecs_realloc_ALREADY_DEFINED
#else
#define yyrealloc grecs_grecs_realloc
#endif

#ifdef yyfree
#define grecs_grecs_free_ALREADY_DEFINED
#else
#define yyfree grecs_grecs_free
#endif

/* %if-c-only */

/* %endif */

/* First, we deal with  platform-specific or compiler-specific issues. */

/* begin standard C headers. */
/* %if-c-only */
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <stdlib.h>
/* %endif */

/* %if-tables-serialization */
/* %endif */
/* end standard C headers. */

/* %if-c-or-c++ */
/* flex integer type definitions */

#ifndef FLEXINT_H
#define FLEXINT_H

/* C99 systems have <inttypes.h>. Non-C99 systems may or may not. */

#if defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L

/* C99 says to define __STDC_LIMIT_MACROS before including stdint.h,
 * if you want the limit (max/min) macros for int types. 
 */
#ifndef __STDC_LIMIT_MACROS
#define __STDC_LIMIT_MACROS 1
#endif

#include <inttypes.h>
typedef int8_t flex_int8_t;
typedef uint8_t flex_uint8_t;
typedef int16_t flex_int16_t;
typedef uint16_t flex_uint16_t;
typedef int32_t flex_int32_t;
typedef uint32_t flex_uint32_t;
#else
typedef signed char flex_int8_t;
typedef short int flex_int16_t;
typedef int flex_int32_t;
typedef unsigned char flex_uint8_t; 
typedef unsigned short int flex_uint16_t;
typedef unsigned int flex_uint32_t;

/* Limits of integral types. */
#ifndef INT8_MIN
#define INT8_MIN               (-128)
#endif
#ifndef INT16_MIN
#define INT16_MIN              (-32767-1)
#endif
#ifndef INT32_MIN
#define INT32_MIN              (-2147483647-1)
#endif
#ifndef INT8_MAX
#define INT8_MAX               (127)
#endif
#ifndef INT16_MAX
#define INT16_MAX              (32767)
#endif
#ifndef INT32_MAX
#define INT32_MAX              (2147483647)
#endif
#ifndef UINT8_MAX
#define UINT8_MAX              (255U)
#endif
#ifndef UINT16_MAX
#define UINT16_MAX             (65535U)
#endif
#ifndef UINT32_MAX
#define UINT32_MAX             (4294967295U)
#endif

#ifndef SIZE_MAX
#define SIZE_MAX               (~(size_t)0)
#endif

#endif /* ! C99 */

#endif /* ! FLEXINT_H */

/* %endif */

/* begin standard C++ headers. */
/* %if-c++-only */
/* %endif */

/* TODO: this is always defined, so inline it */
#define yyconst const

#if defined(__GNUC__) && __GNUC__ >= 3
#define yynoreturn __attribute__((__noreturn__))
#else
#define yynoreturn
#endif

/* %not-for-header */
/* Returned upon end-of-file. */
#define YY_NULL 0
/* %ok-for-header */

/* %not-for-header */
/* Promotes a possibly negative, possibly signed char to an
 *   integer in range [0..255] for use as an array index.
 */
#define YY_SC_TO_UI(c) ((YY_CHAR) (c))
/* %ok-for-header */

/* %if-reentrant */

/* An opaque pointer. */
#ifndef YY_TYPEDEF_YY_SCANNER_T
#define YY_TYPEDEF_YY_SCANNER_T
typedef void* yyscan_t;
#endif

/* For convenience, these vars (plus the bison vars far below)
   are macros in the reentrant scanner. */
#define yyin yyg->yyin_r
#define yyout yyg->yyout_r
#define yyextra yyg->yyextra_r
#define yyleng yyg->yyleng_r
#define yytext yyg->yytext_r
#define yylineno (YY_CURRENT_BUFFER_LVALUE->yy_bs_lineno)
#define yycolumn (YY_CURRENT_BUFFER_LVALUE->yy_bs_column)
#define yy_flex_debug yyg->yy_flex_debug_r

/* %endif */

/* %if-not-reentrant */
/* %endif */

/* Enter a start condition.  This macro really ought to take a parameter,
 * but we do it the disgusting crufty way forced on us by the ()-less
 * definition of BEGIN.
 */
#define BEGIN yyg->yy_start = 1 + 2 *
/* Translate the current start state into a value that can be later handed
 * to BEGIN to return to the state.  The YYSTATE alias is for lex
 * compatibility.
 */
#define YY_START ((yyg->yy_start - 1) / 2)
#define YYSTATE YY_START
/* Action number for EOF rule of a given start state. */
#define YY_STATE_EOF(state) (YY_END_OF_BUFFER + state + 1)
/* Special action meaning "start processing a new file". */
#define YY_NEW_FILE yyrestart( yyin , yyscanner )
#define YY_END_OF_BUFFER_CHAR 0

/* Size of default input buffer. */
#ifndef YY_BUF_SIZE
#ifdef __ia64__
/* On IA-64, the buffer size is 16k, not 8k.
 * Moreover, YY_BUF_SIZE is 2*YY_READ_BUF_SIZE in the general case.
 * Ditto for the __ia64__ case accordingly.
 */
#define YY_BUF_SIZE 32768
#else
#define YY_BUF_SIZE 16384
#endif /* __ia64__ */
#endif

/* The state buf must be large enough to hold one state per character in the main buffer.
 */
#define YY_STATE_BUF_SIZE   ((YY_BUF_SIZE + 2) * sizeof(yy_state_type))

#ifndef YY_TYPEDEF_YY_BUFFER_STATE
#define YY_TYPEDEF_YY_BUFFER_STATE
typedef struct yy_buffer_state *YY_BUFFER_STATE;
#endif

#ifndef YY_TYPEDEF_YY_SIZE_T
#define YY_TYPEDEF_YY_SIZE_T
typedef size_t yy_size_t;
#endif

/* %if-not-reentrant */
/* %endif */

/* %if-c-only */
/* %if-not-reentrant */
/* %endif */
/* %endif */

#define EOB_ACT_CONTINUE_SCAN 0
#define EOB_ACT_END_OF_FILE 1
#define EOB_ACT_LAST_MATCH 2
    
    #define YY_LESS_LINENO(n)
    #define YY_LINENO_REWIND_TO(ptr)
    
/* Return all but the first "n" matched characters back to the input stream. */
#define yyless(n) \
	do \
		{ \
		/* Undo effects of setting up yytext. */ \
        int yyless_macro_arg = (n); \
        YY_LESS_LINENO(yyless_macro_arg);\
		*yy_cp = yyg->yy_hold_char; \
		YY_RESTORE_YY_MORE_OFFSET \
		yyg->yy_c_buf_p = yy_cp = yy_bp + yyless_macro_arg - YY_MORE_ADJ; \
		YY_DO_BEFORE_ACTION; /* set up yytext again */ \
		} \
	while ( 0 )
#define unput(c) yyunput( c, yyg->yytext_ptr , yyscanner )

#ifndef YY_STRUCT_YY_BUFFER_STATE
#define YY_STRUCT_YY_BUFFER_STATE
struct yy_buffer_state
	{
/* %if-c-only */
	FILE *yy_input_file;
/* %endif */

/* %if-c++-only */
/* %endif */

	char *yy_ch_buf;		/* input buffer */
	char *yy_buf_pos;		/* current position in input buffer */

	/* Size of input buffer in bytes, not including room for EOB
	 * characters.
	 */
	int yy_buf_size;

	/* Number of characters read into yy_ch_buf, not including EOB
	 * characters.
	 */
	int yy_n_chars;

	/* Whether we "own" the buffer - i.e., we know we created it,
	 * and can realloc() it to grow it, and should free() it to
	 * delete it.
	 */
	int yy_is_our_buffer;

	/* Whether this is an "interactive" input source; if so, and
	 * if we're using stdio for input, then we want to use getc()
	 * instead of fread(), to make sure we stop fetching input after
	 * each newline.
	 */
	int yy_is_interactive;

	/* Whether we're considered to be at the beginning of a line.
	 * If so, '^' rules will be active on the next match, otherwise
	 * not.
	 */
	int yy_at_bol;

    int yy_bs_lineno; /**< The line count. */
    int yy_bs_column; /**< The column count. */

	/* Whether to try to fill the input buffer when we reach the
	 * end of it.
	 */
	int yy_fill_buffer;

	int yy_buffer_status;

#define YY_BUFFER_NEW 0
#define YY_BUFFER_NORMAL 1
	/* When an EOF's been seen but there's still some text to process
	 * then we mark the buffer as YY_EOF_PENDING, to indicate that we
	 * shouldn't try reading from the input source any more.  We might
	 * still have a bunch of tokens to match, though, because of
	 * possible backing-up.
	 *
	 * When we actually see the EOF, we change the status to "new"
	 * (via yyrestart()), so that the user can continue scanning by
	 * just pointing yyin at a new input file.
	 */
#define YY_BUFFER_EOF_PENDING 2

	};
#endif /* !YY_STRUCT_YY_BUFFER_STATE */

/* %if-c-only Standard (non-C++) definition */
/* %not-for-header */
/* %if-not-reentrant */
/* %endif */
/* %ok-for-header */

/* %endif */

/* We provide macros for accessing buffer states in case in the
 * future we want to put the buffer states in a more general
 * "scanner state".
 *
 * Returns the top of the stack, or NULL.
 */
#define YY_CURRENT_BUFFER ( yyg->yy_buffer_stack \
                          ? yyg->yy_buffer_stack[yyg->yy_buffer_stack_top] \
                          : NULL)
/* Same as previous macro, but useful when we know that the buffer stack is not
 * NULL or when we need an lvalue. For internal use only.
 */
#define YY_CURRENT_BUFFER_LVALUE yyg->yy_buffer_stack[yyg->yy_buffer_stack_top]

/* %if-c-only Standard (non-C++) definition */

/* %if-not-reentrant */
/* %not-for-header */
/* %ok-for-header */

/* %endif */

void yyrestart ( FILE *input_file , yyscan_t yyscanner );
void yy_switch_to_buffer ( YY_BUFFER_STATE new_buffer , yyscan_t yyscanner );
YY_BUFFER_STATE yy_create_buffer ( FILE *file, int size , yyscan_t yyscanner );
void yy_delete_buffer ( YY_BUFFER_STATE b , yyscan_t yyscanner );
void yy_flush_buffer ( YY_BUFFER_STATE b , yyscan_t yyscanner );
void yypush_buffer_state ( YY_BUFFER_STATE new_buffer , yyscan_t yyscanner );
void yypop_buffer_state ( yyscan_t yyscanner );

static void yyensure_buffer_stack ( yyscan_t yyscanner );
static void yy_load_buffer_state ( yyscan_t yyscanner );
static void yy_init_buffer ( YY_BUFFER_STATE b, FILE *file , yyscan_t yyscanner );
#define YY_FLUSH_BUFFER yy_flush_buffer( YY_CURRENT_BUFFER , yyscanner)

YY_BUFFER_STATE yy_scan_buffer ( char *base, yy_size_t size , yyscan_t yyscanner );
YY_BUFFER_STATE yy_scan_string ( const char *yy_str , yyscan_t yyscanner );
YY_BUFFER_STATE yy_scan_bytes ( const char *bytes, int len , yyscan_t yyscanner );

/* %endif */

void *yyalloc ( yy_size_t , yyscan_t yyscanner );
void *yyrealloc ( void *, yy_size_t , yyscan_t yyscanner );
void yyfree ( void * , yyscan_t yyscanner );

#define yy_new_buffer yy_create_buffer
#define yy_set_interactive(is_interactive) \
	{ \
	if ( ! YY_CURRENT_BUFFER ){ \
        yyensure_buffer_stack (yyscanner); \
		YY_CURRENT_BUFFER_LVALUE =    \
            yy_create_buffer( yyin, YY_BUF_SIZE , yyscanner); \
	} \
	YY_CURRENT_BUFFER_LVALUE->yy_is_interactive = is_interactive; \
	}
#define yy_set_bol(at_bol) \
	{ \
	if ( ! YY_CURRENT_BUFFER ){\
        yyensure_buffer_stack (yyscanner); \
		YY_CURRENT_BUFFER_LVALUE =    \
            yy_create_buffer( yyin, YY_BUF_SIZE , yyscanner); \
	} \
	YY_CURRENT_BUFFER_LVALUE->yy_at_bol = at_bol; \
	}
#define YY_AT_BOL() (YY_CURRENT_BUFFER_LVALUE->yy_at_bol)

/* %% [1.0] yytext/yyin/yyout/yy_state_type/yylineno etc. def's & init go here */
/* Begin user sect3 */

#define FLEX_DEBUG
typedef flex_uint8_t YY_CHAR;

typedef int yy_state_type;

#define yytext_ptr yytext_r

/* %% [1.5] DFA */

/* %if-c-only Standard (non-C++) definition */

static yy_state_type yy_get_previous_state ( yyscan_t yyscanner );
static yy_state_type yy_try_NUL_trans ( yy_state_type current_state  , yyscan_t yyscanner);
static int yy_get_next_buffer ( yyscan_t yyscanner );
static void yynoreturn yy_fatal_error ( const char* msg , yyscan_t yyscanner );

/* %endif */

/* Done after the current pattern has been matched and before the
 * corresponding action - sets up yytext.
 */
#define YY_DO_BEFORE_ACTION \
	yyg->yytext_ptr = yy_bp; \
/* %% [2.0] code to fiddle yytext and yyleng for yymore() goes here \ */\
	yyleng = (int) (yy_cp - yy_bp); \
	yyg->yy_hold_char = *yy_cp; \
	*yy_cp = '\0'; \
/* %% [3.0] code to copy yytext_ptr to yytext[] goes here, if %array \ */\
	yyg->yy_c_buf_p = yy_cp;
/* %% [4.0] data tables for the DFA and the user's section 1 definitions go here */
#define YY_NUM_RULES 41
#define YY_END_OF_BUFFER 42
/* This struct is not used in this scanner,
   but its presence is necessary. */
struct yy_trans_info
	{
	flex_int32_t yy_verify;
	flex_int32_t yy_nxt;
	};
static const flex_int16_t yy_accept[262] =
    {   0,
        0,    0,    2,    2,    0,    0,    0,    0,    0,    0,
       42,   16,   37,   38,   40,   13,   40,   39,   17,   17,
       16,   37,   13,    2,    4,    3,   41,   36,   41,   41,
       41,   28,   41,   20,   22,   21,   19,   19,   16,   37,
        0,   23,    0,   13,   13,   12,   17,    1,   15,   16,
       37,    0,   13,   13,   13,   13,   13,   13,   13,    2,
        3,    3,    5,    0,   36,    0,    0,   28,    0,   27,
       26,   20,   19,   19,   18,   25,   24,   15,   14,    0,
       16,   16,   16,    0,    0,    0,    0,    0,    0,    0,
       13,   13,   13,   13,   13,   13,   13,   13,   13,   13,

       13,   13,   13,    0,    0,    0,    0,   31,   16,   16,
        0,    0,    0,    0,    0,    0,    0,    0,   13,   13,
       13,   13,   13,   13,   13,   13,   13,   13,   13,   13,
        0,    0,    0,    0,    0,    0,   16,    0,   29,    0,
        0,    0,    0,    0,    0,    0,   13,   13,   13,   13,
       13,   13,   13,   13,   13,   13,   13,   13,    0,    0,
       34,    0,    0,    0,    0,   30,    0,    0,    0,    0,
        0,    0,    0,   13,   13,   13,   13,   13,   13,    6,
       13,   13,   13,   13,   13,   13,    0,    0,   32,    0,
        0,    6,    0,    0,    0,    0,    0,    0,   13,   13,

       13,   13,   13,   13,   13,   13,    7,   13,    0,   35,
        0,   33,    0,    0,    0,    0,    7,    0,   13,   13,
       13,   11,   13,    9,   13,   13,    0,   11,    0,    9,
        0,    0,   13,   13,   13,   13,    8,   13,   13,    0,
        8,    0,    0,   13,   13,   13,   10,    0,    0,   10,
       13,   13,    0,   13,   13,    0,   13,   13,    0,   13,
        0
    } ;

static const YY_CHAR yy_ec[256] =
    {   0,
        1,    1,    1,    1,    1,    1,    1,    1,    2,    3,
        1,    4,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    5,    1,    6,    7,    1,    1,    1,    8,    9,
        9,   10,    1,    9,   11,    1,   12,   13,   14,   14,
       14,   14,   14,   14,   14,   14,   14,    1,    9,   15,
        1,    1,    1,    1,   16,   16,   16,   16,   16,   16,
       16,   16,   16,   16,   16,   16,   16,   16,   16,   16,
       16,   16,   16,   16,   16,   16,   16,   16,   16,   16,
        1,   17,    1,    1,   18,    1,   19,   20,   21,   22,

       23,   16,   24,   16,   25,   16,   16,   26,   16,   27,
       28,   16,   16,   29,   16,   16,   30,   16,   31,   16,
       16,   16,    9,    1,    9,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,

        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1
    } ;

static const YY_CHAR yy_meta[32] =
    {   0,
        1,    2,    3,    4,    2,    5,    6,    4,    4,    7,
        8,    9,    8,    8,    1,   10,   11,   10,   10,   10,
       10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
       10
    } ;

static const flex_int16_t yy_base[307] =
    {   0,
        0,   14,   22,   24,  918,   28,   27,   33,   49,   60,
      920,    0,    9,  922,   31,   71,  922,  922,    0,   28,
      904,   80,   86,    0,  922,   31,  915,  922,   75,   39,
      922,  922,  914,   37,  922,  922,  922,   87,    0,  102,
       54,  922,  913,  116,  912,  922,    0,  922,  911,  147,
      118,  163,  193,  114,  893,  883,  884,  885,  890,    0,
       98,  103,  922,  905,  922,   99,   97,  922,  904,  922,
      922,  119,  922,  122,  922,  922,  922,  903,  922,    0,
      224,  241,  132,    0,  131,  885,  875,  876,  877,  882,
      257,  123,   64,  143,  147,  137,  173,  178,  877,  870,

      877,  870,  867,  154,  889,  146,  270,  922,  281,  293,
      698,  195,  197,  659,  651,  658,  625,  622,  309,  148,
      156,   25,  163,  164,  181,  623,  621,  620,  621,  616,
      200,  322,  631,  333,  198,  632,  344,  631,  922,  625,
      620,  596,  594,  595,  557,  552,  205,  206,  202,  203,
      218,  211,  230,  556,  548,  526,  245,  509,  217,    0,
      922,  524,  511,  355,  515,  922,  510,  246,  492,  484,
      482,  249,  486,  152,  237,  258,  262,  254,  275,  922,
      296,  299,  301,  468,  482,  456,  478,  477,  922,  476,
      310,  922,  312,  315,  326,  456,  417,  392,  339,  361,

      252,  265,  415,  414,  393,  412,  922,  390,  410,  922,
      409,  922,  408,  407,  386,  405,  922,  383,  316,  273,
      403,  922,  401,  922,  366,  328,  400,  922,  399,  922,
      362,  377,  370,  384,  388,  352,  922,  315,  318,  306,
      922,  304,  277,  323,  242,  260,  922,  334,  224,  922,
      356,  192,  374,  378,  150,  382,  398,    0,  166,   83,
      922,  421,  432,  443,  454,  465,  476,  487,  498,  509,
      520,  531,  542,  553,  564,  575,  586,  597,  608,  619,
       32,  630,  637,  643,  654,  665,  675,  685,  696,  707,
      718,  729,  740,  751,  762,  773,  784,  795,  806,  817,

      828,  839,  850,  861,  872,  883
    } ;

static const flex_int16_t yy_def[307] =
    {   0,
      262,  262,  263,  263,  264,  264,  265,  265,  266,  266,
      261,  267,  261,  261,  268,  269,  261,  261,  270,  270,
      267,  261,  269,  271,  261,  272,  273,  261,  273,  274,
      261,  261,  275,  276,  261,  261,  261,  261,  267,  261,
      268,  261,  277,   23,  278,  261,  270,  261,  279,  280,
      261,  261,  278,   44,   44,   44,   44,   44,   44,  271,
      272,  272,  261,  273,  261,  273,  274,  261,  275,  261,
      261,  276,  261,  261,  261,  261,  261,  279,  261,  281,
      280,  282,   81,   52,  261,  261,  261,  261,  261,  261,
      278,  278,  278,  278,  278,  278,  278,   44,   44,   44,

       44,   44,   44,  273,  283,  284,  261,  261,  285,  282,
      261,  261,  261,  261,  261,  261,  261,  261,  278,  278,
      278,  278,  278,  278,  286,   44,   44,   44,   44,   44,
      273,  261,  283,  287,  284,  288,  285,  288,  261,  289,
      290,  261,  261,  261,  261,  261,  278,  278,  278,  278,
      278,  286,  291,   44,   44,   44,   44,   44,  273,  132,
      261,  292,  261,  287,  289,  261,  290,  293,  261,  261,
      261,  261,  261,  278,  278,  278,  278,  278,  291,  261,
      291,   44,   44,   44,  294,   44,  295,  292,  261,  296,
      293,  261,  293,  261,  261,  261,  297,  261,  278,  278,

      278,  278,  298,  299,   44,  294,  261,   44,  295,  261,
      296,  261,  300,  301,  261,  297,  261,  261,  278,  278,
      298,  261,  299,  261,  302,   44,  300,  261,  301,  261,
      303,  261,  304,  278,  302,  304,  261,  235,  305,  303,
      261,  303,  306,  304,  235,  305,  261,  303,  306,  261,
      304,  235,  303,  304,  235,  303,  304,  235,  303,  304,
        0,  261,  261,  261,  261,  261,  261,  261,  261,  261,
      261,  261,  261,  261,  261,  261,  261,  261,  261,  261,
      261,  261,  261,  261,  261,  261,  261,  261,  261,  261,
      261,  261,  261,  261,  261,  261,  261,  261,  261,  261,

      261,  261,  261,  261,  261,  261
    } ;

static const flex_int16_t yy_nxt[954] =
    {   0,
      261,   13,   14,   13,   13,   15,   16,   17,   18,   19,
       40,   20,   40,   40,   21,   22,   14,   13,   22,   15,
       23,   17,   18,   19,   25,   20,   25,   46,   21,   31,
       28,   26,   32,   26,   29,   31,   42,   48,   32,   49,
       62,  105,   63,   33,   68,  149,   73,   43,   74,   33,
       35,   36,   35,   35,   35,   69,   35,   35,   37,   42,
       38,   35,   36,   35,   35,   35,   46,   35,   35,   37,
       43,   38,   45,   46,   45,   45,   45,   65,   45,   45,
       45,   51,   45,   40,   51,  237,   52,   53,   46,   45,
       53,   45,  121,   45,   45,   45,   75,   45,   75,   54,

       66,   65,   68,   40,   55,   40,   40,  261,   56,  261,
       57,   58,   62,   69,   63,   97,   59,   45,   97,   51,
       45,   40,   51,  104,   52,   46,   98,   98,   73,   44,
       74,   75,  112,   75,   44,  112,  261,  261,   44,   46,
       44,   44,  120,  113,  113,   46,   44,   39,   39,   46,
       46,   80,   80,   39,   46,  124,   65,   81,   46,   39,
       39,   39,  135,   83,   84,   46,   46,   84,  241,  122,
      147,  123,  258,  199,   97,   46,   85,   97,  125,   97,
      131,   86,   97,   46,  148,   87,  153,   88,   89,  150,
       98,   98,  151,   90,   53,   46,  112,   53,  112,  112,

      141,  112,   65,  261,   46,   46,   91,   46,   46,  113,
      113,   92,  255,   46,  261,   93,  153,   94,   95,   65,
       46,  187,  159,   96,   39,  177,  250,  176,  106,   80,
       39,  174,  180,  175,   39,  181,   39,   39,   39,   46,
       83,   39,  107,  108,  178,  107,  185,  109,  192,  185,
      197,  193,  111,  197,   46,   39,   46,   39,   97,   46,
       46,   97,  247,  185,   46,  200,  185,   46,  252,  119,
      119,  107,  108,  219,  107,   46,  136,  180,  202,  250,
      181,  111,  138,  139,  138,  138,  138,  201,  138,  138,
      138,  220,  138,   39,  107,  108,  234,  107,  180,  109,

      203,  181,  204,  203,  111,  204,  241,   39,  241,   39,
       97,   46,  192,   97,  192,  193,  213,  193,   46,  213,
      247,  119,  119,  160,  161,  237,  160,  214,  162,  239,
      214,  248,  239,  163,  107,  108,  241,  107,  233,  136,
      203,   46,  245,  203,  111,  138,  139,  138,  138,  138,
      251,  138,  138,  138,  237,  138,  107,  108,  237,  107,
      253,  136,  204,   46,  241,  204,  111,  236,  237,  236,
      236,  236,  237,  236,  236,  236,  241,  236,  243,  242,
      237,  243,  254,  238,  241,  239,   46,  244,  239,  236,
      237,  236,  236,  236,  256,  236,  236,  236,  257,  236,

      237,  230,  228,  224,  259,  222,  232,  217,  231,  230,
      228,  212,  210,  226,  207,  225,  224,  222,  218,  217,
      260,   12,   12,   12,   12,   12,   12,   12,   12,   12,
       12,   12,   24,   24,   24,   24,   24,   24,   24,   24,
       24,   24,   24,   27,   27,   27,   27,   27,   27,   27,
       27,   27,   27,   27,   30,   30,   30,   30,   30,   30,
       30,   30,   30,   30,   30,   34,   34,   34,   34,   34,
       34,   34,   34,   34,   34,   34,   39,  215,  212,  189,
      210,   39,  208,   39,  207,   39,   39,   41,   41,  205,
       41,   41,   41,   41,   41,   41,   41,   41,   44,   44,

       44,   44,   44,   44,   44,   44,   44,   44,   44,   47,
      198,  196,  195,  194,   47,  168,   47,  166,   47,   47,
       60,   60,  190,   60,   60,   60,  189,   60,   60,   60,
       60,   61,   61,  186,   61,   61,   61,   61,   61,   61,
       61,   61,   64,   64,   64,   64,   64,   64,   64,   64,
       64,   64,   64,   67,   67,  184,   67,   67,   67,   67,
       67,   67,   67,   67,   70,   70,   70,   70,   70,   70,
       70,   70,   70,   70,   70,   72,  183,  182,  173,  172,
       72,   72,   72,   72,   72,   72,   76,   76,   76,   76,
       76,   76,   76,   76,   76,   76,   76,   45,   45,   45,

       45,   45,   45,   45,   45,   45,   45,   45,   78,   78,
       78,   78,   78,   78,   78,   78,   78,   78,   78,   82,
      171,  170,  169,   82,   82,  168,   82,  166,   82,   82,
      110,  110,  110,  139,  139,  110,  132,  110,  110,  110,
      110,  133,  158,  157,  133,  156,  133,  134,  155,  154,
      146,  145,  134,  134,  137,  137,  137,  137,  137,  137,
      137,  137,  137,  137,  137,  152,  152,  152,  152,  152,
      152,  152,  152,  152,  152,  152,  164,  164,  144,  143,
      164,  142,  164,  164,  164,  138,  138,  138,  138,  138,
      138,  138,  138,  138,  138,  138,  165,  165,  165,  165,

      165,  165,  165,  165,  165,  165,  165,  167,  167,  140,
      167,  167,  167,  167,  167,  167,  167,  167,  179,  179,
      179,  179,  179,  179,  179,  179,  179,  179,  179,  188,
      188,  188,  188,  188,  188,  188,  188,  188,  188,  188,
      191,  191,  191,  191,  191,  191,  191,  191,  191,  191,
      191,  206,  206,  206,  206,  206,  206,  206,  206,  206,
      206,  206,  209,  209,  209,  209,  209,  209,  209,  209,
      209,  209,  209,  211,  211,  211,  211,  211,  211,  211,
      211,  211,  211,  211,  216,  216,  216,  216,  216,  216,
      216,  216,  216,  216,  216,  221,  221,  221,  221,  221,

      221,  221,  221,  221,  221,  221,  223,  223,  223,  223,
      223,  223,  223,  223,  223,  223,  223,  227,  227,  227,
      227,  227,  227,  227,  227,  227,  227,  227,  229,  229,
      229,  229,  229,  229,  229,  229,  229,  229,  229,  235,
      235,  235,  235,  235,  235,  235,  235,  235,  235,  235,
      240,  240,  240,  240,  240,  240,  240,  240,  240,  240,
      240,  236,  236,  236,  236,  236,  236,  236,  236,  236,
      236,  236,  246,  246,  246,  246,  246,  246,  246,  246,
      246,  246,  246,  249,  249,  249,  249,  249,  249,  249,
      249,  249,  249,  249,  132,  130,  129,  128,  127,  126,

      118,  117,  116,  115,  114,   79,   71,   65,  103,  102,
      101,  100,   99,   79,   46,   77,   71,   65,   50,  261,
       28,   11,  261,  261,  261,  261,  261,  261,  261,  261,
      261,  261,  261,  261,  261,  261,  261,  261,  261,  261,
      261,  261,  261,  261,  261,  261,  261,  261,  261,  261,
      261,  261,  261
    } ;

static const flex_int16_t yy_chk[954] =
    {   0,
        0,    1,    1,    1,    1,    1,    1,    1,    1,    1,
       13,    1,   13,   13,    1,    2,    2,    2,    2,    2,
        2,    2,    2,    2,    3,    2,    4,  122,    2,    7,
        6,    3,    7,    4,    6,    8,   15,   20,    8,   20,
       26,  281,   26,    7,   30,  122,   34,   15,   34,    8,
        9,    9,    9,    9,    9,   30,    9,    9,    9,   41,
        9,   10,   10,   10,   10,   10,   93,   10,   10,   10,
       41,   10,   16,   16,   16,   16,   16,   29,   16,   16,
       16,   22,   16,   22,   22,  260,   22,   23,   23,   23,
       23,   23,   93,   23,   23,   23,   38,   23,   38,   23,

       29,   66,   67,   40,   23,   40,   40,   61,   23,   61,
       23,   23,   62,   67,   62,   54,   23,   44,   54,   51,
       44,   51,   51,   66,   51,   92,   54,   54,   72,   44,
       72,   74,   85,   74,   44,   85,   83,   83,   44,   96,
       44,   44,   92,   85,   85,   94,   44,   50,   83,   95,
      120,  106,   50,   50,  174,   96,  104,   50,  121,   50,
       50,   50,  106,   50,   52,  123,  124,   52,  259,   94,
      120,   95,  255,  174,   97,   97,   52,   97,   97,   98,
      104,   52,   98,  125,  121,   52,  125,   52,   52,  123,
       98,   98,  124,   52,   53,   53,  112,   53,  113,  112,

      112,  113,  131,  135,  149,  150,   53,  147,  148,  113,
      113,   53,  252,  152,  135,   53,  152,   53,   53,  159,
      151,  159,  131,   53,   81,  150,  249,  149,   81,   81,
       81,  147,  153,  148,   81,  153,   81,   81,   81,  175,
       81,   82,   82,   82,  151,   82,  157,   82,  168,  157,
      172,  168,   82,  172,  201,   82,  178,   82,   91,   91,
      176,   91,  246,  177,  177,  175,  177,  202,  245,   91,
       91,  107,  107,  201,  107,  220,  107,  179,  178,  243,
      179,  107,  109,  109,  109,  109,  109,  176,  109,  109,
      109,  202,  109,  110,  110,  110,  220,  110,  181,  110,

      182,  181,  183,  182,  110,  183,  242,  110,  240,  110,
      119,  119,  191,  119,  193,  191,  194,  193,  219,  194,
      239,  119,  119,  132,  132,  244,  132,  195,  132,  226,
      195,  242,  226,  132,  134,  134,  248,  134,  219,  134,
      199,  199,  238,  199,  134,  137,  137,  137,  137,  137,
      244,  137,  137,  137,  236,  137,  164,  164,  251,  164,
      248,  164,  200,  200,  231,  200,  164,  225,  225,  225,
      225,  225,  233,  225,  225,  225,  253,  225,  232,  231,
      254,  232,  251,  225,  256,  234,  234,  233,  234,  235,
      235,  235,  235,  235,  253,  235,  235,  235,  254,  235,

      257,  229,  227,  223,  256,  221,  218,  216,  215,  214,
      213,  211,  209,  208,  206,  205,  204,  203,  198,  197,
      257,  262,  262,  262,  262,  262,  262,  262,  262,  262,
      262,  262,  263,  263,  263,  263,  263,  263,  263,  263,
      263,  263,  263,  264,  264,  264,  264,  264,  264,  264,
      264,  264,  264,  264,  265,  265,  265,  265,  265,  265,
      265,  265,  265,  265,  265,  266,  266,  266,  266,  266,
      266,  266,  266,  266,  266,  266,  267,  196,  190,  188,
      187,  267,  186,  267,  185,  267,  267,  268,  268,  184,
      268,  268,  268,  268,  268,  268,  268,  268,  269,  269,

      269,  269,  269,  269,  269,  269,  269,  269,  269,  270,
      173,  171,  170,  169,  270,  167,  270,  165,  270,  270,
      271,  271,  163,  271,  271,  271,  162,  271,  271,  271,
      271,  272,  272,  158,  272,  272,  272,  272,  272,  272,
      272,  272,  273,  273,  273,  273,  273,  273,  273,  273,
      273,  273,  273,  274,  274,  156,  274,  274,  274,  274,
      274,  274,  274,  274,  275,  275,  275,  275,  275,  275,
      275,  275,  275,  275,  275,  276,  155,  154,  146,  145,
      276,  276,  276,  276,  276,  276,  277,  277,  277,  277,
      277,  277,  277,  277,  277,  277,  277,  278,  278,  278,

      278,  278,  278,  278,  278,  278,  278,  278,  279,  279,
      279,  279,  279,  279,  279,  279,  279,  279,  279,  280,
      144,  143,  142,  280,  280,  141,  280,  140,  280,  280,
      282,  282,  282,  138,  136,  282,  133,  282,  282,  282,
      282,  283,  130,  129,  283,  128,  283,  284,  127,  126,
      118,  117,  284,  284,  285,  285,  285,  285,  285,  285,
      285,  285,  285,  285,  285,  286,  286,  286,  286,  286,
      286,  286,  286,  286,  286,  286,  287,  287,  116,  115,
      287,  114,  287,  287,  287,  288,  288,  288,  288,  288,
      288,  288,  288,  288,  288,  288,  289,  289,  289,  289,

      289,  289,  289,  289,  289,  289,  289,  290,  290,  111,
      290,  290,  290,  290,  290,  290,  290,  290,  291,  291,
      291,  291,  291,  291,  291,  291,  291,  291,  291,  292,
      292,  292,  292,  292,  292,  292,  292,  292,  292,  292,
      293,  293,  293,  293,  293,  293,  293,  293,  293,  293,
      293,  294,  294,  294,  294,  294,  294,  294,  294,  294,
      294,  294,  295,  295,  295,  295,  295,  295,  295,  295,
      295,  295,  295,  296,  296,  296,  296,  296,  296,  296,
      296,  296,  296,  296,  297,  297,  297,  297,  297,  297,
      297,  297,  297,  297,  297,  298,  298,  298,  298,  298,

      298,  298,  298,  298,  298,  298,  299,  299,  299,  299,
      299,  299,  299,  299,  299,  299,  299,  300,  300,  300,
      300,  300,  300,  300,  300,  300,  300,  300,  301,  301,
      301,  301,  301,  301,  301,  301,  301,  301,  301,  302,
      302,  302,  302,  302,  302,  302,  302,  302,  302,  302,
      303,  303,  303,  303,  303,  303,  303,  303,  303,  303,
      303,  304,  304,  304,  304,  304,  304,  304,  304,  304,
      304,  304,  305,  305,  305,  305,  305,  305,  305,  305,
      305,  305,  305,  306,  306,  306,  306,  306,  306,  306,
      306,  306,  306,  306,  105,  103,  102,  101,  100,   99,

       90,   89,   88,   87,   86,   78,   69,   64,   59,   58,
       57,   56,   55,   49,   45,   43,   33,   27,   21,   11,
        5,  261,  261,  261,  261,  261,  261,  261,  261,  261,
      261,  261,  261,  261,  261,  261,  261,  261,  261,  261,
      261,  261,  261,  261,  261,  261,  261,  261,  261,  261,
      261,  261,  261
    } ;

static const flex_int16_t yy_rule_linenum[41] =
    {   0,
      160,  166,  167,  168,  171,  190,  194,  200,  202,  206,
      210,  217,  219,  220,  223,  226,  232,  239,  247,  251,
      258,  266,  276,  284,  292,  300,  305,  309,  324,  325,
      326,  327,  328,  329,  336,  339,  373,  375,  379,  383
    } ;

/* The intent behind this definition is that it'll catch
 * any uses of REJECT which flex missed.
 */
#define REJECT reject_used_but_not_detected
#define yymore() yymore_used_but_not_detected
#define YY_MORE_ADJ 0
#define YY_RESTORE_YY_MORE_OFFSET
#line 1 "grecs-lex.l"
/* grecs - Gray's Extensible Configuration System       -*- c -*- */

#line 12 "grecs-lex.l"
/* grecs - Gray's Extensible Configuration System
   Copyright (C) 2007-2025 Sergey Poznyakoff

   Grecs is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 3 of the License, or (at your
   option) any later version.

   Grecs is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with Grecs. If not, see <http://www.gnu.org/licenses/>. */

#include <grecs.h>
#include <grecs-gram.h>
#include <unistd.h>
#include <fcntl.h>
#include <ctype.h>
#include <stdlib.h>
#include <errno.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <glob.h>
#include <signal.h>
#include <wordsplit.h>

#define PCTL (yyget_extra (yyscanner))

/*
 * Locus tracking
 */
#define YY_USER_ACTION do {						\
		if (YYSTATE == INITIAL) {				\
			yylloc->beg = PCTL->point;			\
			yylloc->beg.col++;				\
		}							\
		PCTL->point.col += yyleng;				\
		yylloc->end = PCTL->point;				\
	} while (0);

#define YYSTYPE         GRECS_GRECS_STYPE

/*
 * Redefine default error reporting function.
 */
#define YY_FATAL_ERROR(t) grecs_error(NULL, errno, "%s", t)

/*
 * Preprocessing and synclines.
 */
static void echo(yyscan_t yyscanner, char const *text, size_t len);

/* Inform flex that it should use echo. */
#define ECHO echo(yyscanner, yytext, yyleng)
#define PPECHO do { if (PCTL->preproc_mode) ECHO; } while(0)

/*
 * String concatenation support.
 *
 * String concatenation is a legacy feature of Grecs parser that instructs
 * it to concatenate adjacent quoted strings.  It is enabled if the
 * GRECS_OPTION_QUOTED_STRING_CONCAT bit is set in parser options.  This
 * bit causes the lexer to qualify each double-quoted string as QSTRING.
 */
#define qstring()							\
	((grecs_parser_options & GRECS_OPTION_QUOTED_STRING_CONCAT)	\
	 ? QSTRING : STRING)

/*
 * Lexical tie-in variables.
 */
#define grecs_yyerror_suppress PCTL->error_suppress

/*
 * Here-document support.
 */
static int (*char_to_strip)(char);      /* Strips matching characters of each
					   here-document line */

static void multiline_begin(yyscan_t scanner, char *);
static void multiline_add(yyscan_t scanner, char *);
static char *multiline_strip_tabs(char *text);
static int isemptystr(char const *text, int off);

/*
 * Forward declarations.
 */
static int string_or_ident(char const *);
static void qstring_locus_fixup(yyscan_t scanner);
static int parse_include(yyscan_t scanner, char *text);
static int push_source(yyscan_t scanner, const char *name, int once);
static int pop_source(yyscan_t scanner);
static void ctx_abort(yyscan_t scanner);

/*
 * Versions of grecs up to 03044f768f didn't recognize comment starters that
 * followed a non-quoted string constituent, as in inet://127.0.0.1.  That
 * silently encouraged a bad practice of omitting quotes around such strings.
 *
 * To discontinue such practice and point to the source of the problem, a
 * warning is emitted each time a comment starter is encountered immediately
 * following a non-quoted string constituent.
 */
static void comment_warning(yyscan_t scanner);

/*
 * The report_diag should be called only when yytext begins with the following
 * prefix:
 *       ^[ \t]*#[ \t]*.{WLEN}
 * It returns a pointer to yytext past the prefix.
 */
static char *
report_diag(char *text, size_t leng, size_t wlen)
{
	char *p = strchr(text, '#');
	p += strspn(p, " \t") + wlen;
	p += strspn(p, " \t");
	text[leng-1] = 0;
	if (p[0] == '"' && text[leng-2] == '"') {
		char *q, *r;

		text[leng-2] = 0;
		p++;
		q = r = p;
		do {
			if (*r == '\\') {
				int c = wordsplit_c_unquote_char(*++r);
				*q = c ? c : *r;
			} else
				*q = *r++;
		} while (*q++ == 0);
	}
	return p;
}
#line 1188 "grecs-lex.c"

#line 1190 "grecs-lex.c"

#define INITIAL 0
#define COMMENT 1
#define ML 2
#define STR 3
#define NQSTR 4

#ifndef YY_NO_UNISTD_H
/* Special case for "unistd.h", since it is non-ANSI. We include it way
 * down here because we want the user's section 1 to have been scanned first.
 * The user has a chance to override it with an option.
 */
/* %if-c-only */
#include <unistd.h>
/* %endif */
/* %if-c++-only */
/* %endif */
#endif

#define YY_EXTRA_TYPE struct parser_control *

/* %if-c-only Reentrant structure and macros (non-C++). */
/* %if-reentrant */

/* Holds the entire state of the reentrant scanner. */
struct yyguts_t
    {

    /* User-defined. Not touched by flex. */
    YY_EXTRA_TYPE yyextra_r;

    /* The rest are the same as the globals declared in the non-reentrant scanner. */
    FILE *yyin_r, *yyout_r;
    size_t yy_buffer_stack_top; /**< index of top of stack. */
    size_t yy_buffer_stack_max; /**< capacity of stack. */
    YY_BUFFER_STATE * yy_buffer_stack; /**< Stack as an array. */
    char yy_hold_char;
    int yy_n_chars;
    int yyleng_r;
    char *yy_c_buf_p;
    int yy_init;
    int yy_start;
    int yy_did_buffer_switch_on_eof;
    int yy_start_stack_ptr;
    int yy_start_stack_depth;
    int *yy_start_stack;
    yy_state_type yy_last_accepting_state;
    char* yy_last_accepting_cpos;

    int yylineno_r;
    int yy_flex_debug_r;

    char *yytext_r;
    int yy_more_flag;
    int yy_more_len;

    YYSTYPE * yylval_r;

    YYLTYPE * yylloc_r;

    }; /* end struct yyguts_t */

/* %if-c-only */

static int yy_init_globals ( yyscan_t yyscanner );

/* %endif */

/* %if-reentrant */

    /* This must go here because YYSTYPE and YYLTYPE are included
     * from bison output in section 1.*/
    #    define yylval yyg->yylval_r
    
    #    define yylloc yyg->yylloc_r
    
int yylex_init (yyscan_t* scanner);

int yylex_init_extra ( YY_EXTRA_TYPE user_defined, yyscan_t* scanner);

/* %endif */

/* %endif End reentrant structures and macros. */

/* Accessor methods to globals.
   These are made visible to non-reentrant scanners for convenience. */

int yylex_destroy ( yyscan_t yyscanner );

int yyget_debug ( yyscan_t yyscanner );

void yyset_debug ( int debug_flag , yyscan_t yyscanner );

YY_EXTRA_TYPE yyget_extra ( yyscan_t yyscanner );

void yyset_extra ( YY_EXTRA_TYPE user_defined , yyscan_t yyscanner );

FILE *yyget_in ( yyscan_t yyscanner );

void yyset_in  ( FILE * _in_str , yyscan_t yyscanner );

FILE *yyget_out ( yyscan_t yyscanner );

void yyset_out  ( FILE * _out_str , yyscan_t yyscanner );

			int yyget_leng ( yyscan_t yyscanner );

char *yyget_text ( yyscan_t yyscanner );

int yyget_lineno ( yyscan_t yyscanner );

void yyset_lineno ( int _line_number , yyscan_t yyscanner );

int yyget_column  ( yyscan_t yyscanner );

void yyset_column ( int _column_no , yyscan_t yyscanner );

/* %if-bison-bridge */

YYSTYPE * yyget_lval ( yyscan_t yyscanner );

void yyset_lval ( YYSTYPE * yylval_param , yyscan_t yyscanner );

       YYLTYPE *yyget_lloc ( yyscan_t yyscanner );
    
        void yyset_lloc ( YYLTYPE * yylloc_param , yyscan_t yyscanner );
    
/* %endif */

/* Macros after this point can all be overridden by user definitions in
 * section 1.
 */

#ifndef YY_SKIP_YYWRAP
#ifdef __cplusplus
extern "C" int yywrap ( yyscan_t yyscanner );
#else
extern int yywrap ( yyscan_t yyscanner );
#endif
#endif

/* %not-for-header */
#ifndef YY_NO_UNPUT
    
#endif
/* %ok-for-header */

/* %endif */

#ifndef yytext_ptr
static void yy_flex_strncpy ( char *, const char *, int , yyscan_t yyscanner);
#endif

#ifdef YY_NEED_STRLEN
static int yy_flex_strlen ( const char * , yyscan_t yyscanner);
#endif

#ifndef YY_NO_INPUT
/* %if-c-only Standard (non-C++) definition */
/* %not-for-header */
#ifdef __cplusplus
static int yyinput ( yyscan_t yyscanner );
#else
static int input ( yyscan_t yyscanner );
#endif
/* %ok-for-header */

/* %endif */
#endif

/* %if-c-only */

/* %endif */

/* Amount of stuff to slurp up with each read. */
#ifndef YY_READ_BUF_SIZE
#ifdef __ia64__
/* On IA-64, the buffer size is 16k, not 8k */
#define YY_READ_BUF_SIZE 16384
#else
#define YY_READ_BUF_SIZE 8192
#endif /* __ia64__ */
#endif

/* Copy whatever the last rule matched to the standard output. */
#ifndef ECHO
/* %if-c-only Standard (non-C++) definition */
/* This used to be an fputs(), but since the string might contain NUL's,
 * we now use fwrite().
 */
#define ECHO do { if (fwrite( yytext, (size_t) yyleng, 1, yyout )) {} } while (0)
/* %endif */
/* %if-c++-only C++ definition */
/* %endif */
#endif

/* Gets input and stuffs it into "buf".  number of characters read, or YY_NULL,
 * is returned in "result".
 */
#ifndef YY_INPUT
#define YY_INPUT(buf,result,max_size) \
/* %% [5.0] fread()/read() definition of YY_INPUT goes here unless we're doing C++ \ */\
	if ( YY_CURRENT_BUFFER_LVALUE->yy_is_interactive ) \
		{ \
		int c = '*'; \
		int n; \
		for ( n = 0; n < max_size && \
			     (c = getc( yyin )) != EOF && c != '\n'; ++n ) \
			buf[n] = (char) c; \
		if ( c == '\n' ) \
			buf[n++] = (char) c; \
		if ( c == EOF && ferror( yyin ) ) \
			YY_FATAL_ERROR( "input in flex scanner failed" ); \
		result = n; \
		} \
	else \
		{ \
		errno=0; \
		while ( (result = (int) fread(buf, 1, (yy_size_t) max_size, yyin)) == 0 && ferror(yyin)) \
			{ \
			if( errno != EINTR) \
				{ \
				YY_FATAL_ERROR( "input in flex scanner failed" ); \
				break; \
				} \
			errno=0; \
			clearerr(yyin); \
			} \
		}\
\
/* %if-c++-only C++ definition \ */\
/* %endif */

#endif

/* No semi-colon after return; correct usage is to write "yyterminate();" -
 * we don't want an extra ';' after the "return" because that will cause
 * some compilers to complain about unreachable statements.
 */
#ifndef yyterminate
#define yyterminate() return YY_NULL
#endif

/* Number of entries by which start-condition stack grows. */
#ifndef YY_START_STACK_INCR
#define YY_START_STACK_INCR 25
#endif

/* Report a fatal error. */
#ifndef YY_FATAL_ERROR
/* %if-c-only */
#define YY_FATAL_ERROR(msg) yy_fatal_error( msg , yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */
#endif

/* %if-tables-serialization structures and prototypes */
/* %not-for-header */
/* %ok-for-header */

/* %not-for-header */
/* %tables-yydmap generated elements */
/* %endif */
/* end tables serialization structures and prototypes */

/* %ok-for-header */

/* Default declaration of generated scanner - a define so the user can
 * easily add parameters.
 */
#ifndef YY_DECL
#define YY_DECL_IS_OURS 1
/* %if-c-only Standard (non-C++) definition */

extern int yylex \
               (YYSTYPE * yylval_param, YYLTYPE * yylloc_param , yyscan_t yyscanner);

#define YY_DECL int yylex \
               (YYSTYPE * yylval_param, YYLTYPE * yylloc_param , yyscan_t yyscanner)
/* %endif */
/* %if-c++-only C++ definition */
/* %endif */
#endif /* !YY_DECL */

/* Code executed at the beginning of each rule, after yytext and yyleng
 * have been set up.
 */
#ifndef YY_USER_ACTION
#define YY_USER_ACTION
#endif

/* Code executed at the end of each rule. */
#ifndef YY_BREAK
#define YY_BREAK /*LINTED*/break;
#endif

/* %% [6.0] YY_RULE_SETUP definition goes here */
#define YY_RULE_SETUP \
	if ( yyleng > 0 ) \
		YY_CURRENT_BUFFER_LVALUE->yy_at_bol = \
				(yytext[yyleng - 1] == '\n'); \
	YY_USER_ACTION

/* %not-for-header */
/** The main scanner function which does all the work.
 */
YY_DECL
{
	yy_state_type yy_current_state;
	char *yy_cp, *yy_bp;
	int yy_act;
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

    yylval = yylval_param;

    yylloc = yylloc_param;

	if ( !yyg->yy_init )
		{
		yyg->yy_init = 1;

#ifdef YY_USER_INIT
		YY_USER_INIT;
#endif

		if ( ! yyg->yy_start )
			yyg->yy_start = 1;	/* first start state */

		if ( ! yyin )
/* %if-c-only */
			yyin = stdin;
/* %endif */
/* %if-c++-only */
/* %endif */

		if ( ! yyout )
/* %if-c-only */
			yyout = stdout;
/* %endif */
/* %if-c++-only */
/* %endif */

		if ( ! YY_CURRENT_BUFFER ) {
			yyensure_buffer_stack (yyscanner);
			YY_CURRENT_BUFFER_LVALUE =
				yy_create_buffer( yyin, YY_BUF_SIZE , yyscanner);
		}

		yy_load_buffer_state( yyscanner );
		}

	{
/* %% [7.0] user's declarations go here */
#line 157 "grecs-lex.l"

#line 159 "grecs-lex.l"
	 /* C-style comments */
#line 1549 "grecs-lex.c"

	while ( /*CONSTCOND*/1 )		/* loops until end-of-file is reached */
		{
/* %% [8.0] yymore()-related code goes here */
		yy_cp = yyg->yy_c_buf_p;

		/* Support of yytext. */
		*yy_cp = yyg->yy_hold_char;

		/* yy_bp points to the position in yy_ch_buf of the start of
		 * the current run.
		 */
		yy_bp = yy_cp;

/* %% [9.0] code to set up and find next match goes here */
		yy_current_state = yyg->yy_start;
		yy_current_state += YY_AT_BOL();
yy_match:
		do
			{
			YY_CHAR yy_c = yy_ec[YY_SC_TO_UI(*yy_cp)] ;
			if ( yy_accept[yy_current_state] )
				{
				yyg->yy_last_accepting_state = yy_current_state;
				yyg->yy_last_accepting_cpos = yy_cp;
				}
			while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
				{
				yy_current_state = (int) yy_def[yy_current_state];
				if ( yy_current_state >= 262 )
					yy_c = yy_meta[yy_c];
				}
			yy_current_state = yy_nxt[yy_base[yy_current_state] + yy_c];
			++yy_cp;
			}
		while ( yy_base[yy_current_state] != 922 );

yy_find_action:
/* %% [10.0] code to find the action number goes here */
		yy_act = yy_accept[yy_current_state];
		if ( yy_act == 0 )
			{ /* have to back up */
			yy_cp = yyg->yy_last_accepting_cpos;
			yy_current_state = yyg->yy_last_accepting_state;
			yy_act = yy_accept[yy_current_state];
			}

		YY_DO_BEFORE_ACTION;

/* %% [11.0] code for yylineno update goes here */

do_action:	/* This label is used only to access EOF actions. */

/* %% [12.0] debug code goes here */
		if ( yy_flex_debug )
			{
			if ( yy_act == 0 )
				fprintf( stderr, "--scanner backing up\n" );
			else if ( yy_act < 41 )
				fprintf( stderr, "--accepting rule at line %ld (\"%s\")\n",
				         (long)yy_rule_linenum[yy_act], yytext );
			else if ( yy_act == 41 )
				fprintf( stderr, "--accepting default rule (\"%s\")\n",
				         yytext );
			else if ( yy_act == 42 )
				fprintf( stderr, "--(end of buffer or a NUL)\n" );
			else
				fprintf( stderr, "--EOF (start condition %d)\n", YY_START );
			}

		switch ( yy_act )
	{ /* beginning of action switch */
/* %% [13.0] actions go here */
			case 0: /* must back up */
			/* undo the effects of YY_DO_BEFORE_ACTION */
			*yy_cp = yyg->yy_hold_char;
			yy_cp = yyg->yy_last_accepting_cpos;
			yy_current_state = yyg->yy_last_accepting_state;
			goto yy_find_action;

case 1:
YY_RULE_SETUP
#line 160 "grecs-lex.l"
{
	    BEGIN(COMMENT);
	    PCTL->beginning_of_line = 0;
	    comment_warning(yyscanner);
 }
	YY_BREAK

case 2:
YY_RULE_SETUP
#line 166 "grecs-lex.l"
/* eat anything that's not a '*' */
	YY_BREAK
case 3:
YY_RULE_SETUP
#line 167 "grecs-lex.l"
/* eat up '*'s not followed by '/'s */
	YY_BREAK
case 4:
/* rule 4 can match eol */
YY_RULE_SETUP
#line 168 "grecs-lex.l"
{ struct parser_control *pctl = yyget_extra(yyscanner);
	      grecs_locus_point_advance_line(pctl->point);
	      pctl->emit_syncline = 1; }
	YY_BREAK
case 5:
YY_RULE_SETUP
#line 171 "grecs-lex.l"
{
	      struct parser_control *pctl = yyget_extra(yyscanner);
	      BEGIN(INITIAL);
	      if (pctl->preproc_mode) {
		      if (yylloc->end.line != yylloc->beg.line) {
			      if (!(yylloc->beg.line == 1 &&
				    yylloc->beg.col == 1))
				      echo(yyscanner, "\n", 1);
			      pctl->beginning_of_line = 1;
		      } else
			      pctl->beginning_of_line = 0;
	      }
 }
	YY_BREAK
case YY_STATE_EOF(COMMENT):
#line 184 "grecs-lex.l"
{ grecs_error(yylloc, 0, _("end of file in comment"));
	      return 0;
 }
	YY_BREAK

/* Line directive */
case 6:
/* rule 6 can match eol */
YY_RULE_SETUP
#line 190 "grecs-lex.l"
{
	 PPECHO;
	 grecs_parse_line_directive_cpp(yytext, yylloc, &PCTL->point);
 }
	YY_BREAK
case 7:
/* rule 7 can match eol */
YY_RULE_SETUP
#line 194 "grecs-lex.l"
{
	 PPECHO;
	 grecs_parse_line_directive(yytext, yylloc, &PCTL->point);
 }
	YY_BREAK
/* Includes */
case 8:
/* rule 8 can match eol */
*yy_cp = yyg->yy_hold_char; /* undo effects of setting up yytext */
YY_LINENO_REWIND_TO(yy_cp - 1);
yyg->yy_c_buf_p = yy_cp -= 1;
YY_DO_BEFORE_ACTION; /* set up yytext again */
YY_RULE_SETUP
#line 200 "grecs-lex.l"
{ parse_include(yyscanner, yytext); }
	YY_BREAK
/* Error directives */
case 9:
/* rule 9 can match eol */
YY_RULE_SETUP
#line 202 "grecs-lex.l"
{
	grecs_error(yylloc, 0, "%s", report_diag(yytext, yyleng,
						  sizeof("error")));
 }
	YY_BREAK
case 10:
/* rule 10 can match eol */
YY_RULE_SETUP
#line 206 "grecs-lex.l"
{
  grecs_warning(yylloc, 0, "%s", report_diag(yytext, yyleng,
					      sizeof("warning")));
 }
	YY_BREAK
case 11:
/* rule 11 can match eol */
YY_RULE_SETUP
#line 210 "grecs-lex.l"
{
  grecs_error(yylloc, 0, "%s", report_diag(yytext, yyleng, sizeof("abend")));
  ctx_abort(yyscanner);
  grecs_yyerror_suppress = 1;
  return 0;
 }
	YY_BREAK
/* End-of-line comments */
case 12:
/* rule 12 can match eol */
YY_RULE_SETUP
#line 217 "grecs-lex.l"
{ grecs_locus_point_advance_line(PCTL->point);
	    PCTL->emit_syncline = 1; }
	YY_BREAK
case 13:
YY_RULE_SETUP
#line 219 "grecs-lex.l"
/* end-of-file comment */;
	YY_BREAK
case 14:
/* rule 14 can match eol */
YY_RULE_SETUP
#line 220 "grecs-lex.l"
{ comment_warning(yyscanner);
	    grecs_locus_point_advance_line(PCTL->point);
	    PCTL->emit_syncline = 1; }
	YY_BREAK
case 15:
YY_RULE_SETUP
#line 223 "grecs-lex.l"
{ comment_warning(yyscanner); } /* end of file comment */
	YY_BREAK
/* Non-quoted strings and identifiers */
case 16:
YY_RULE_SETUP
#line 226 "grecs-lex.l"
{
	 PPECHO;
	 grecs_line_begin(&PCTL->acc);
	 grecs_line_add(PCTL->acc, yytext, yyleng);
	 BEGIN(NQSTR);
 }
	YY_BREAK
case 17:
YY_RULE_SETUP
#line 232 "grecs-lex.l"
{
	 PPECHO;
	 grecs_line_begin(&PCTL->acc);
	 grecs_line_add(PCTL->acc, yytext, yyleng);
	 BEGIN(NQSTR);
 }
	YY_BREAK

case 18:
*yy_cp = yyg->yy_hold_char; /* undo effects of setting up yytext */
yyg->yy_c_buf_p = yy_cp -= 2;
YY_DO_BEFORE_ACTION; /* set up yytext again */
YY_RULE_SETUP
#line 239 "grecs-lex.l"
{
	 PPECHO;
	 PCTL->emit_comment_warning = 1;
	 grecs_line_add(PCTL->acc, yytext, yyleng);
	 yylval->string = grecs_line_finish(PCTL->acc);
	 BEGIN(INITIAL);
	 return STRING;
 }
	YY_BREAK
case 19:
YY_RULE_SETUP
#line 247 "grecs-lex.l"
{
	 PPECHO;
	 grecs_line_add(PCTL->acc, yytext, yyleng);
 }
	YY_BREAK
case 20:
YY_RULE_SETUP
#line 251 "grecs-lex.l"
{
	 PPECHO;
	 grecs_line_add(PCTL->acc, yytext, yyleng);
	 yylval->string = grecs_line_finish(PCTL->acc);
	 BEGIN(INITIAL);
	 return string_or_ident(yylval->string);
 }
	YY_BREAK
case 21:
/* rule 21 can match eol */
YY_RULE_SETUP
#line 258 "grecs-lex.l"
{
	 yyless(0);
	 yylloc->end.col--;
	 PCTL->point.col--;
	 yylval->string = grecs_line_finish(PCTL->acc);
	 BEGIN(INITIAL);
	 return string_or_ident(yylval->string);
 }
	YY_BREAK
case 22:
YY_RULE_SETUP
#line 266 "grecs-lex.l"
{
	 yyless(0);
	 yylloc->end.col--;
	 PCTL->point.col--;
	 yylval->string = grecs_line_finish(PCTL->acc);
	 BEGIN(INITIAL);
	 return string_or_ident(yylval->string);
 }
	YY_BREAK

/* Quoted strings */
case 23:
YY_RULE_SETUP
#line 276 "grecs-lex.l"
{
	 PPECHO;
	 grecs_line_begin(&PCTL->acc);
	 grecs_line_add(PCTL->acc, yytext + 1, yyleng - 2);
	 yylval->string = grecs_line_finish(PCTL->acc);
	 qstring_locus_fixup(yyscanner);
	 return qstring();
 }
	YY_BREAK
case 24:
/* rule 24 can match eol */
YY_RULE_SETUP
#line 284 "grecs-lex.l"
{
	 PPECHO;
	 BEGIN(STR);
	 grecs_line_begin(&PCTL->acc);
	 grecs_line_acc_grow_unescape_last(PCTL->acc,
					   yytext + 1, yyleng - 1, yylloc);
	 grecs_locus_point_advance_line(PCTL->point);
 }
	YY_BREAK
case 25:
YY_RULE_SETUP
#line 292 "grecs-lex.l"
{
	 PPECHO;
	 BEGIN(STR);
	 grecs_line_begin(&PCTL->acc);
	 grecs_line_acc_grow_unescape_last(PCTL->acc,
					   yytext + 1, yyleng - 1, yylloc);
 }
	YY_BREAK

case 26:
/* rule 26 can match eol */
YY_RULE_SETUP
#line 300 "grecs-lex.l"
{
	 PPECHO;
	 grecs_line_acc_grow_unescape_last(PCTL->acc, yytext, yyleng, yylloc);
	 grecs_locus_point_advance_line(PCTL->point);
 }
	YY_BREAK
case 27:
YY_RULE_SETUP
#line 305 "grecs-lex.l"
{
	 PPECHO;
	 grecs_line_acc_grow_unescape_last(PCTL->acc, yytext, yyleng, yylloc);
 }
	YY_BREAK
case 28:
YY_RULE_SETUP
#line 309 "grecs-lex.l"
{
	 PPECHO;
	 BEGIN(INITIAL);
	 if (yyleng > 1)
		 grecs_line_add(PCTL->acc, yytext, yyleng - 1);
	 yylval->string = grecs_line_finish(PCTL->acc);
	 qstring_locus_fixup(yyscanner);
	 return qstring();
 }
	YY_BREAK
case YY_STATE_EOF(STR):
#line 318 "grecs-lex.l"
{
	grecs_error(yylloc, 0, _("end of file in quoted string"));
	return 0;
 }
	YY_BREAK

/* Multiline strings */
case 29:
/* rule 29 can match eol */
#line 325 "grecs-lex.l"
case 30:
/* rule 30 can match eol */
#line 326 "grecs-lex.l"
case 31:
/* rule 31 can match eol */
#line 327 "grecs-lex.l"
case 32:
/* rule 32 can match eol */
#line 328 "grecs-lex.l"
case 33:
/* rule 33 can match eol */
#line 329 "grecs-lex.l"
case 34:
/* rule 34 can match eol */
YY_RULE_SETUP
#line 329 "grecs-lex.l"
{
	 PPECHO;
	 BEGIN(ML);
	 multiline_begin(yyscanner, yytext+2);
 }
	YY_BREAK

/* Ignore m4 line statements */
case 35:
/* rule 35 can match eol */
YY_RULE_SETUP
#line 336 "grecs-lex.l"
{
	grecs_locus_point_advance_line(PCTL->point);
}
	YY_BREAK
case 36:
/* rule 36 can match eol */
YY_RULE_SETUP
#line 339 "grecs-lex.l"
{  struct parser_control *pctl = yyget_extra(yyscanner);
	 char *p = multiline_strip_tabs(yytext);
	 int len = p + pctl->multiline_delimiter_len - yytext;
	 if (!strncmp(p, pctl->multiline_delimiter,
		      pctl->multiline_delimiter_len)
	     && isemptystr(yytext, len)) {
		 /* Update end pos */
		 if (grecs_parser_options & GRECS_OPTION_ADJUST_STRING_LOCATIONS)
			 yylloc->end.col = 1;
		 else
			 yylloc->end.col -= yyleng - len;
		 /*
		  * Adjust current position and put the characters that
		  * follow the delimiter back to input.
		  */
		 pctl->point.col -= yyleng - len;
		 yyless(len);

		 /* Echo the consumed input if preprocessing. */
		 PPECHO;

		 grecs_free(pctl->multiline_delimiter);
		 pctl->multiline_delimiter = NULL;
		 BEGIN(INITIAL);
		 yylval->string = grecs_line_finish(pctl->acc);

		 return MSTRING;
	 } else
		 PPECHO;
	 grecs_locus_point_advance_line(pctl->point);
	 multiline_add(yyscanner, p);
  }
	YY_BREAK

case 37:
YY_RULE_SETUP
#line 373 "grecs-lex.l"
PPECHO;
	YY_BREAK
/* Other tokens */
case 38:
/* rule 38 can match eol */
YY_RULE_SETUP
#line 375 "grecs-lex.l"
{
	 PPECHO;
	 grecs_locus_point_advance_line(PCTL->point);
 }
	YY_BREAK
case 39:
YY_RULE_SETUP
#line 379 "grecs-lex.l"
{
	 PPECHO;
	 return yytext[0];
 }
	YY_BREAK
case 40:
YY_RULE_SETUP
#line 383 "grecs-lex.l"
{ if (isascii(yytext[0]) && isprint(yytext[0]))
		grecs_error(yylloc, 0,
			     _("stray character %c"), yytext[0]);
	   else
		grecs_error(yylloc, 0,
			     _("stray character \\%03o"),
			       (unsigned char) yytext[0]); }
	YY_BREAK
case 41:
YY_RULE_SETUP
#line 391 "grecs-lex.l"
ECHO;
	YY_BREAK
#line 2040 "grecs-lex.c"
case YY_STATE_EOF(INITIAL):
case YY_STATE_EOF(ML):
case YY_STATE_EOF(NQSTR):
	yyterminate();

	case YY_END_OF_BUFFER:
		{
		/* Amount of text matched not including the EOB char. */
		int yy_amount_of_matched_text = (int) (yy_cp - yyg->yytext_ptr) - 1;

		/* Undo the effects of YY_DO_BEFORE_ACTION. */
		*yy_cp = yyg->yy_hold_char;
		YY_RESTORE_YY_MORE_OFFSET

		if ( YY_CURRENT_BUFFER_LVALUE->yy_buffer_status == YY_BUFFER_NEW )
			{
			/* We're scanning a new file or input source.  It's
			 * possible that this happened because the user
			 * just pointed yyin at a new source and called
			 * yylex().  If so, then we have to assure
			 * consistency between YY_CURRENT_BUFFER and our
			 * globals.  Here is the right place to do so, because
			 * this is the first action (other than possibly a
			 * back-up) that will match for the new input source.
			 */
			yyg->yy_n_chars = YY_CURRENT_BUFFER_LVALUE->yy_n_chars;
/* %if-c-only */
			YY_CURRENT_BUFFER_LVALUE->yy_input_file = yyin;
/* %endif */
/* %if-c++-only */
/* %endif */
			YY_CURRENT_BUFFER_LVALUE->yy_buffer_status = YY_BUFFER_NORMAL;
			}

		/* Note that here we test for yy_c_buf_p "<=" to the position
		 * of the first EOB in the buffer, since yy_c_buf_p will
		 * already have been incremented past the NUL character
		 * (since all states make transitions on EOB to the
		 * end-of-buffer state).  Contrast this with the test
		 * in input().
		 */
		if ( yyg->yy_c_buf_p <= &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[yyg->yy_n_chars] )
			{ /* This was really a NUL. */
			yy_state_type yy_next_state;

			yyg->yy_c_buf_p = yyg->yytext_ptr + yy_amount_of_matched_text;

			yy_current_state = yy_get_previous_state( yyscanner );

			/* Okay, we're now positioned to make the NUL
			 * transition.  We couldn't have
			 * yy_get_previous_state() go ahead and do it
			 * for us because it doesn't know how to deal
			 * with the possibility of jamming (and we don't
			 * want to build jamming into it because then it
			 * will run more slowly).
			 */

			yy_next_state = yy_try_NUL_trans( yy_current_state , yyscanner);

			yy_bp = yyg->yytext_ptr + YY_MORE_ADJ;

			if ( yy_next_state )
				{
				/* Consume the NUL. */
				yy_cp = ++yyg->yy_c_buf_p;
				yy_current_state = yy_next_state;
				goto yy_match;
				}

			else
				{
/* %% [14.0] code to do back-up for compressed tables and set up yy_cp goes here */
				yy_cp = yyg->yy_c_buf_p;
				goto yy_find_action;
				}
			}

		else switch ( yy_get_next_buffer( yyscanner ) )
			{
			case EOB_ACT_END_OF_FILE:
				{
				yyg->yy_did_buffer_switch_on_eof = 0;

				if ( yywrap( yyscanner ) )
					{
					/* Note: because we've taken care in
					 * yy_get_next_buffer() to have set up
					 * yytext, we can now set up
					 * yy_c_buf_p so that if some total
					 * hoser (like flex itself) wants to
					 * call the scanner after we return the
					 * YY_NULL, it'll still work - another
					 * YY_NULL will get returned.
					 */
					yyg->yy_c_buf_p = yyg->yytext_ptr + YY_MORE_ADJ;

					yy_act = YY_STATE_EOF(YY_START);
					goto do_action;
					}

				else
					{
					if ( ! yyg->yy_did_buffer_switch_on_eof )
						YY_NEW_FILE;
					}
				break;
				}

			case EOB_ACT_CONTINUE_SCAN:
				yyg->yy_c_buf_p =
					yyg->yytext_ptr + yy_amount_of_matched_text;

				yy_current_state = yy_get_previous_state( yyscanner );

				yy_cp = yyg->yy_c_buf_p;
				yy_bp = yyg->yytext_ptr + YY_MORE_ADJ;
				goto yy_match;

			case EOB_ACT_LAST_MATCH:
				yyg->yy_c_buf_p =
				&YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[yyg->yy_n_chars];

				yy_current_state = yy_get_previous_state( yyscanner );

				yy_cp = yyg->yy_c_buf_p;
				yy_bp = yyg->yytext_ptr + YY_MORE_ADJ;
				goto yy_find_action;
			}
		break;
		}

	default:
		YY_FATAL_ERROR(
			"fatal flex scanner internal error--no action found" );
	} /* end of action switch */
		} /* end of scanning one token */
	} /* end of user's declarations */
} /* end of yylex */
/* %ok-for-header */

/* %if-c++-only */
/* %not-for-header */
/* %ok-for-header */

/* %endif */

/* yy_get_next_buffer - try to read in a new buffer
 *
 * Returns a code representing an action:
 *	EOB_ACT_LAST_MATCH -
 *	EOB_ACT_CONTINUE_SCAN - continue scanning from current position
 *	EOB_ACT_END_OF_FILE - end of file
 */
/* %if-c-only */
static int yy_get_next_buffer (yyscan_t yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
	char *dest = YY_CURRENT_BUFFER_LVALUE->yy_ch_buf;
	char *source = yyg->yytext_ptr;
	int number_to_move, i;
	int ret_val;

	if ( yyg->yy_c_buf_p > &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[yyg->yy_n_chars + 1] )
		YY_FATAL_ERROR(
		"fatal flex scanner internal error--end of buffer missed" );

	if ( YY_CURRENT_BUFFER_LVALUE->yy_fill_buffer == 0 )
		{ /* Don't try to fill the buffer, so this is an EOF. */
		if ( yyg->yy_c_buf_p - yyg->yytext_ptr - YY_MORE_ADJ == 1 )
			{
			/* We matched a single character, the EOB, so
			 * treat this as a final EOF.
			 */
			return EOB_ACT_END_OF_FILE;
			}

		else
			{
			/* We matched some text prior to the EOB, first
			 * process it.
			 */
			return EOB_ACT_LAST_MATCH;
			}
		}

	/* Try to read more data. */

	/* First move last chars to start of buffer. */
	number_to_move = (int) (yyg->yy_c_buf_p - yyg->yytext_ptr - 1);

	for ( i = 0; i < number_to_move; ++i )
		*(dest++) = *(source++);

	if ( YY_CURRENT_BUFFER_LVALUE->yy_buffer_status == YY_BUFFER_EOF_PENDING )
		/* don't do the read, it's not guaranteed to return an EOF,
		 * just force an EOF
		 */
		YY_CURRENT_BUFFER_LVALUE->yy_n_chars = yyg->yy_n_chars = 0;

	else
		{
			int num_to_read =
			YY_CURRENT_BUFFER_LVALUE->yy_buf_size - number_to_move - 1;

		while ( num_to_read <= 0 )
			{ /* Not enough room in the buffer - grow it. */

			/* just a shorter name for the current buffer */
			YY_BUFFER_STATE b = YY_CURRENT_BUFFER_LVALUE;

			int yy_c_buf_p_offset =
				(int) (yyg->yy_c_buf_p - b->yy_ch_buf);

			if ( b->yy_is_our_buffer )
				{
				int new_size = b->yy_buf_size * 2;

				if ( new_size <= 0 )
					b->yy_buf_size += b->yy_buf_size / 8;
				else
					b->yy_buf_size *= 2;

				b->yy_ch_buf = (char *)
					/* Include room in for 2 EOB chars. */
					yyrealloc( (void *) b->yy_ch_buf,
							 (yy_size_t) (b->yy_buf_size + 2) , yyscanner );
				}
			else
				/* Can't grow it, we don't own it. */
				b->yy_ch_buf = NULL;

			if ( ! b->yy_ch_buf )
				YY_FATAL_ERROR(
				"fatal error - scanner input buffer overflow" );

			yyg->yy_c_buf_p = &b->yy_ch_buf[yy_c_buf_p_offset];

			num_to_read = YY_CURRENT_BUFFER_LVALUE->yy_buf_size -
						number_to_move - 1;

			}

		if ( num_to_read > YY_READ_BUF_SIZE )
			num_to_read = YY_READ_BUF_SIZE;

		/* Read in more data. */
		YY_INPUT( (&YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[number_to_move]),
			yyg->yy_n_chars, num_to_read );

		YY_CURRENT_BUFFER_LVALUE->yy_n_chars = yyg->yy_n_chars;
		}

	if ( yyg->yy_n_chars == 0 )
		{
		if ( number_to_move == YY_MORE_ADJ )
			{
			ret_val = EOB_ACT_END_OF_FILE;
			yyrestart( yyin  , yyscanner);
			}

		else
			{
			ret_val = EOB_ACT_LAST_MATCH;
			YY_CURRENT_BUFFER_LVALUE->yy_buffer_status =
				YY_BUFFER_EOF_PENDING;
			}
		}

	else
		ret_val = EOB_ACT_CONTINUE_SCAN;

	if ((yyg->yy_n_chars + number_to_move) > YY_CURRENT_BUFFER_LVALUE->yy_buf_size) {
		/* Extend the array by 50%, plus the number we really need. */
		int new_size = yyg->yy_n_chars + number_to_move + (yyg->yy_n_chars >> 1);
		YY_CURRENT_BUFFER_LVALUE->yy_ch_buf = (char *) yyrealloc(
			(void *) YY_CURRENT_BUFFER_LVALUE->yy_ch_buf, (yy_size_t) new_size , yyscanner );
		if ( ! YY_CURRENT_BUFFER_LVALUE->yy_ch_buf )
			YY_FATAL_ERROR( "out of dynamic memory in yy_get_next_buffer()" );
		/* "- 2" to take care of EOB's */
		YY_CURRENT_BUFFER_LVALUE->yy_buf_size = (int) (new_size - 2);
	}

	yyg->yy_n_chars += number_to_move;
	YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[yyg->yy_n_chars] = YY_END_OF_BUFFER_CHAR;
	YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[yyg->yy_n_chars + 1] = YY_END_OF_BUFFER_CHAR;

	yyg->yytext_ptr = &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[0];

	return ret_val;
}

/* yy_get_previous_state - get the state just before the EOB char was reached */

/* %if-c-only */
/* %not-for-header */
    static yy_state_type yy_get_previous_state (yyscan_t yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */
{
	yy_state_type yy_current_state;
	char *yy_cp;
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

/* %% [15.0] code to get the start state into yy_current_state goes here */
	yy_current_state = yyg->yy_start;
	yy_current_state += YY_AT_BOL();

	for ( yy_cp = yyg->yytext_ptr + YY_MORE_ADJ; yy_cp < yyg->yy_c_buf_p; ++yy_cp )
		{
/* %% [16.0] code to find the next state goes here */
		YY_CHAR yy_c = (*yy_cp ? yy_ec[YY_SC_TO_UI(*yy_cp)] : 1);
		if ( yy_accept[yy_current_state] )
			{
			yyg->yy_last_accepting_state = yy_current_state;
			yyg->yy_last_accepting_cpos = yy_cp;
			}
		while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
			{
			yy_current_state = (int) yy_def[yy_current_state];
			if ( yy_current_state >= 262 )
				yy_c = yy_meta[yy_c];
			}
		yy_current_state = yy_nxt[yy_base[yy_current_state] + yy_c];
		}

	return yy_current_state;
}

/* yy_try_NUL_trans - try to make a transition on the NUL character
 *
 * synopsis
 *	next_state = yy_try_NUL_trans( current_state );
 */
/* %if-c-only */
    static yy_state_type yy_try_NUL_trans  (yy_state_type yy_current_state , yyscan_t yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */
{
	int yy_is_jam;
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner; /* This var may be unused depending upon options. */
/* %% [17.0] code to find the next state, and perhaps do backing up, goes here */
	char *yy_cp = yyg->yy_c_buf_p;

	YY_CHAR yy_c = 1;
	if ( yy_accept[yy_current_state] )
		{
		yyg->yy_last_accepting_state = yy_current_state;
		yyg->yy_last_accepting_cpos = yy_cp;
		}
	while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
		{
		yy_current_state = (int) yy_def[yy_current_state];
		if ( yy_current_state >= 262 )
			yy_c = yy_meta[yy_c];
		}
	yy_current_state = yy_nxt[yy_base[yy_current_state] + yy_c];
	yy_is_jam = (yy_current_state == 261);

	(void)yyg;
	return yy_is_jam ? 0 : yy_current_state;
}

#ifndef YY_NO_UNPUT
/* %if-c-only */

/* %endif */
#endif

/* %if-c-only */
#ifndef YY_NO_INPUT
#ifdef __cplusplus
    static int yyinput (yyscan_t yyscanner)
#else
    static int input  (yyscan_t yyscanner)
#endif

/* %endif */
/* %if-c++-only */
/* %endif */
{
	int c;
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

	*yyg->yy_c_buf_p = yyg->yy_hold_char;

	if ( *yyg->yy_c_buf_p == YY_END_OF_BUFFER_CHAR )
		{
		/* yy_c_buf_p now points to the character we want to return.
		 * If this occurs *before* the EOB characters, then it's a
		 * valid NUL; if not, then we've hit the end of the buffer.
		 */
		if ( yyg->yy_c_buf_p < &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[yyg->yy_n_chars] )
			/* This was really a NUL. */
			*yyg->yy_c_buf_p = '\0';

		else
			{ /* need more input */
			int offset = (int) (yyg->yy_c_buf_p - yyg->yytext_ptr);
			++yyg->yy_c_buf_p;

			switch ( yy_get_next_buffer( yyscanner ) )
				{
				case EOB_ACT_LAST_MATCH:
					/* This happens because yy_g_n_b()
					 * sees that we've accumulated a
					 * token and flags that we need to
					 * try matching the token before
					 * proceeding.  But for input(),
					 * there's no matching to consider.
					 * So convert the EOB_ACT_LAST_MATCH
					 * to EOB_ACT_END_OF_FILE.
					 */

					/* Reset buffer status. */
					yyrestart( yyin , yyscanner);

					/*FALLTHROUGH*/

				case EOB_ACT_END_OF_FILE:
					{
					if ( yywrap( yyscanner ) )
						return 0;

					if ( ! yyg->yy_did_buffer_switch_on_eof )
						YY_NEW_FILE;
#ifdef __cplusplus
					return yyinput(yyscanner);
#else
					return input(yyscanner);
#endif
					}

				case EOB_ACT_CONTINUE_SCAN:
					yyg->yy_c_buf_p = yyg->yytext_ptr + offset;
					break;
				}
			}
		}

	c = *(unsigned char *) yyg->yy_c_buf_p;	/* cast for 8-bit char's */
	*yyg->yy_c_buf_p = '\0';	/* preserve yytext */
	yyg->yy_hold_char = *++yyg->yy_c_buf_p;

/* %% [19.0] update BOL and yylineno */
	YY_CURRENT_BUFFER_LVALUE->yy_at_bol = (c == '\n');

	return c;
}
/* %if-c-only */
#endif	/* ifndef YY_NO_INPUT */
/* %endif */

/** Immediately switch to a different input stream.
 * @param input_file A readable stream.
 * @param yyscanner The scanner object.
 * @note This function does not reset the start condition to @c INITIAL .
 */
/* %if-c-only */
    void yyrestart  (FILE * input_file , yyscan_t yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

	if ( ! YY_CURRENT_BUFFER ){
        yyensure_buffer_stack (yyscanner);
		YY_CURRENT_BUFFER_LVALUE =
            yy_create_buffer( yyin, YY_BUF_SIZE , yyscanner);
	}

	yy_init_buffer( YY_CURRENT_BUFFER, input_file , yyscanner);
	yy_load_buffer_state( yyscanner );
}

/* %if-c++-only */
/* %endif */

/** Switch to a different input buffer.
 * @param new_buffer The new input buffer.
 * @param yyscanner The scanner object.
 */
/* %if-c-only */
    void yy_switch_to_buffer  (YY_BUFFER_STATE  new_buffer , yyscan_t yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

	/* TODO. We should be able to replace this entire function body
	 * with
	 *		yypop_buffer_state();
	 *		yypush_buffer_state(new_buffer);
     */
	yyensure_buffer_stack (yyscanner);
	if ( YY_CURRENT_BUFFER == new_buffer )
		return;

	if ( YY_CURRENT_BUFFER )
		{
		/* Flush out information for old buffer. */
		*yyg->yy_c_buf_p = yyg->yy_hold_char;
		YY_CURRENT_BUFFER_LVALUE->yy_buf_pos = yyg->yy_c_buf_p;
		YY_CURRENT_BUFFER_LVALUE->yy_n_chars = yyg->yy_n_chars;
		}

	YY_CURRENT_BUFFER_LVALUE = new_buffer;
	yy_load_buffer_state( yyscanner );

	/* We don't actually know whether we did this switch during
	 * EOF (yywrap()) processing, but the only time this flag
	 * is looked at is after yywrap() is called, so it's safe
	 * to go ahead and always set it.
	 */
	yyg->yy_did_buffer_switch_on_eof = 1;
}

/* %if-c-only */
static void yy_load_buffer_state  (yyscan_t yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
	yyg->yy_n_chars = YY_CURRENT_BUFFER_LVALUE->yy_n_chars;
	yyg->yytext_ptr = yyg->yy_c_buf_p = YY_CURRENT_BUFFER_LVALUE->yy_buf_pos;
/* %if-c-only */
	yyin = YY_CURRENT_BUFFER_LVALUE->yy_input_file;
/* %endif */
/* %if-c++-only */
/* %endif */
	yyg->yy_hold_char = *yyg->yy_c_buf_p;
}

/** Allocate and initialize an input buffer state.
 * @param file A readable stream.
 * @param size The character buffer size in bytes. When in doubt, use @c YY_BUF_SIZE.
 * @param yyscanner The scanner object.
 * @return the allocated buffer state.
 */
/* %if-c-only */
    YY_BUFFER_STATE yy_create_buffer  (FILE * file, int  size , yyscan_t yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */
{
	YY_BUFFER_STATE b;
    
	b = (YY_BUFFER_STATE) yyalloc( sizeof( struct yy_buffer_state ) , yyscanner );
	if ( ! b )
		YY_FATAL_ERROR( "out of dynamic memory in yy_create_buffer()" );

	b->yy_buf_size = size;

	/* yy_ch_buf has to be 2 characters longer than the size given because
	 * we need to put in 2 end-of-buffer characters.
	 */
	b->yy_ch_buf = (char *) yyalloc( (yy_size_t) (b->yy_buf_size + 2) , yyscanner );
	if ( ! b->yy_ch_buf )
		YY_FATAL_ERROR( "out of dynamic memory in yy_create_buffer()" );

	b->yy_is_our_buffer = 1;

	yy_init_buffer( b, file , yyscanner);

	return b;
}

/* %if-c++-only */
/* %endif */

/** Destroy the buffer.
 * @param b a buffer created with yy_create_buffer()
 * @param yyscanner The scanner object.
 */
/* %if-c-only */
    void yy_delete_buffer (YY_BUFFER_STATE  b , yyscan_t yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

	if ( ! b )
		return;

	if ( b == YY_CURRENT_BUFFER ) /* Not sure if we should pop here. */
		YY_CURRENT_BUFFER_LVALUE = (YY_BUFFER_STATE) 0;

	if ( b->yy_is_our_buffer )
		yyfree( (void *) b->yy_ch_buf , yyscanner );

	yyfree( (void *) b , yyscanner );
}

/* Initializes or reinitializes a buffer.
 * This function is sometimes called more than once on the same buffer,
 * such as during a yyrestart() or at EOF.
 */
/* %if-c-only */
    static void yy_init_buffer  (YY_BUFFER_STATE  b, FILE * file , yyscan_t yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */

{
	int oerrno = errno;
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

	yy_flush_buffer( b , yyscanner);

/* %if-c-only */
	b->yy_input_file = file;
/* %endif */
/* %if-c++-only */
/* %endif */
	b->yy_fill_buffer = 1;

    /* If b is the current buffer, then yy_init_buffer was _probably_
     * called from yyrestart() or through yy_get_next_buffer.
     * In that case, we don't want to reset the lineno or column.
     */
    if (b != YY_CURRENT_BUFFER){
        b->yy_bs_lineno = 1;
        b->yy_bs_column = 0;
    }

/* %if-c-only */

        b->yy_is_interactive = file ? (isatty( fileno(file) ) > 0) : 0;
    
/* %endif */
/* %if-c++-only */
/* %endif */
	errno = oerrno;
}

/** Discard all buffered characters. On the next scan, YY_INPUT will be called.
 * @param b the buffer state to be flushed, usually @c YY_CURRENT_BUFFER.
 * @param yyscanner The scanner object.
 */
/* %if-c-only */
    void yy_flush_buffer (YY_BUFFER_STATE  b , yyscan_t yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
	if ( ! b )
		return;

	b->yy_n_chars = 0;

	/* We always need two end-of-buffer characters.  The first causes
	 * a transition to the end-of-buffer state.  The second causes
	 * a jam in that state.
	 */
	b->yy_ch_buf[0] = YY_END_OF_BUFFER_CHAR;
	b->yy_ch_buf[1] = YY_END_OF_BUFFER_CHAR;

	b->yy_buf_pos = &b->yy_ch_buf[0];

	b->yy_at_bol = 1;
	b->yy_buffer_status = YY_BUFFER_NEW;

	if ( b == YY_CURRENT_BUFFER )
		yy_load_buffer_state( yyscanner );
}

/* %if-c-or-c++ */
/** Pushes the new state onto the stack. The new state becomes
 *  the current state. This function will allocate the stack
 *  if necessary.
 *  @param new_buffer The new state.
 *  @param yyscanner The scanner object.
 */
/* %if-c-only */
void yypush_buffer_state (YY_BUFFER_STATE new_buffer , yyscan_t yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
	if (new_buffer == NULL)
		return;

	yyensure_buffer_stack(yyscanner);

	/* This block is copied from yy_switch_to_buffer. */
	if ( YY_CURRENT_BUFFER )
		{
		/* Flush out information for old buffer. */
		*yyg->yy_c_buf_p = yyg->yy_hold_char;
		YY_CURRENT_BUFFER_LVALUE->yy_buf_pos = yyg->yy_c_buf_p;
		YY_CURRENT_BUFFER_LVALUE->yy_n_chars = yyg->yy_n_chars;
		}

	/* Only push if top exists. Otherwise, replace top. */
	if (YY_CURRENT_BUFFER)
		yyg->yy_buffer_stack_top++;
	YY_CURRENT_BUFFER_LVALUE = new_buffer;

	/* copied from yy_switch_to_buffer. */
	yy_load_buffer_state( yyscanner );
	yyg->yy_did_buffer_switch_on_eof = 1;
}
/* %endif */

/* %if-c-or-c++ */
/** Removes and deletes the top of the stack, if present.
 *  The next element becomes the new top.
 *  @param yyscanner The scanner object.
 */
/* %if-c-only */
void yypop_buffer_state (yyscan_t yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
	if (!YY_CURRENT_BUFFER)
		return;

	yy_delete_buffer(YY_CURRENT_BUFFER , yyscanner);
	YY_CURRENT_BUFFER_LVALUE = NULL;
	if (yyg->yy_buffer_stack_top > 0)
		--yyg->yy_buffer_stack_top;

	if (YY_CURRENT_BUFFER) {
		yy_load_buffer_state( yyscanner );
		yyg->yy_did_buffer_switch_on_eof = 1;
	}
}
/* %endif */

/* %if-c-or-c++ */
/* Allocates the stack if it does not exist.
 *  Guarantees space for at least one push.
 */
/* %if-c-only */
static void yyensure_buffer_stack (yyscan_t yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */
{
	yy_size_t num_to_alloc;
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

	if (!yyg->yy_buffer_stack) {

		/* First allocation is just for 2 elements, since we don't know if this
		 * scanner will even need a stack. We use 2 instead of 1 to avoid an
		 * immediate realloc on the next call.
         */
      num_to_alloc = 1; /* After all that talk, this was set to 1 anyways... */
		yyg->yy_buffer_stack = (struct yy_buffer_state**)yyalloc
								(num_to_alloc * sizeof(struct yy_buffer_state*)
								, yyscanner);
		if ( ! yyg->yy_buffer_stack )
			YY_FATAL_ERROR( "out of dynamic memory in yyensure_buffer_stack()" );

		memset(yyg->yy_buffer_stack, 0, num_to_alloc * sizeof(struct yy_buffer_state*));

		yyg->yy_buffer_stack_max = num_to_alloc;
		yyg->yy_buffer_stack_top = 0;
		return;
	}

	if (yyg->yy_buffer_stack_top >= (yyg->yy_buffer_stack_max) - 1){

		/* Increase the buffer to prepare for a possible push. */
		yy_size_t grow_size = 8 /* arbitrary grow size */;

		num_to_alloc = yyg->yy_buffer_stack_max + grow_size;
		yyg->yy_buffer_stack = (struct yy_buffer_state**)yyrealloc
								(yyg->yy_buffer_stack,
								num_to_alloc * sizeof(struct yy_buffer_state*)
								, yyscanner);
		if ( ! yyg->yy_buffer_stack )
			YY_FATAL_ERROR( "out of dynamic memory in yyensure_buffer_stack()" );

		/* zero only the new slots.*/
		memset(yyg->yy_buffer_stack + yyg->yy_buffer_stack_max, 0, grow_size * sizeof(struct yy_buffer_state*));
		yyg->yy_buffer_stack_max = num_to_alloc;
	}
}
/* %endif */

/* %if-c-only */
/** Setup the input buffer state to scan directly from a user-specified character buffer.
 * @param base the character buffer
 * @param size the size in bytes of the character buffer
 * @param yyscanner The scanner object.
 * @return the newly allocated buffer state object.
 */
YY_BUFFER_STATE yy_scan_buffer  (char * base, yy_size_t  size , yyscan_t yyscanner)
{
	YY_BUFFER_STATE b;
    
	if ( size < 2 ||
	     base[size-2] != YY_END_OF_BUFFER_CHAR ||
	     base[size-1] != YY_END_OF_BUFFER_CHAR )
		/* They forgot to leave room for the EOB's. */
		return NULL;

	b = (YY_BUFFER_STATE) yyalloc( sizeof( struct yy_buffer_state ) , yyscanner );
	if ( ! b )
		YY_FATAL_ERROR( "out of dynamic memory in yy_scan_buffer()" );

	b->yy_buf_size = (int) (size - 2);	/* "- 2" to take care of EOB's */
	b->yy_buf_pos = b->yy_ch_buf = base;
	b->yy_is_our_buffer = 0;
	b->yy_input_file = NULL;
	b->yy_n_chars = b->yy_buf_size;
	b->yy_is_interactive = 0;
	b->yy_at_bol = 1;
	b->yy_fill_buffer = 0;
	b->yy_buffer_status = YY_BUFFER_NEW;

	yy_switch_to_buffer( b , yyscanner );

	return b;
}
/* %endif */

/* %if-c-only */
/** Setup the input buffer state to scan a string. The next call to yylex() will
 * scan from a @e copy of @a str.
 * @param yystr a NUL-terminated string to scan
 * @param yyscanner The scanner object.
 * @return the newly allocated buffer state object.
 * @note If you want to scan bytes that may contain NUL values, then use
 *       yy_scan_bytes() instead.
 */
YY_BUFFER_STATE yy_scan_string (const char * yystr , yyscan_t yyscanner)
{
    
	return yy_scan_bytes( yystr, (int) strlen(yystr) , yyscanner);
}
/* %endif */

/* %if-c-only */
/** Setup the input buffer state to scan the given bytes. The next call to yylex() will
 * scan from a @e copy of @a bytes.
 * @param yybytes the byte buffer to scan
 * @param _yybytes_len the number of bytes in the buffer pointed to by @a bytes.
 * @param yyscanner The scanner object.
 * @return the newly allocated buffer state object.
 */
YY_BUFFER_STATE yy_scan_bytes  (const char * yybytes, int  _yybytes_len , yyscan_t yyscanner)
{
	YY_BUFFER_STATE b;
	char *buf;
	yy_size_t n;
	int i;
    
	/* Get memory for full buffer, including space for trailing EOB's. */
	n = (yy_size_t) (_yybytes_len + 2);
	buf = (char *) yyalloc( n , yyscanner );
	if ( ! buf )
		YY_FATAL_ERROR( "out of dynamic memory in yy_scan_bytes()" );

	for ( i = 0; i < _yybytes_len; ++i )
		buf[i] = yybytes[i];

	buf[_yybytes_len] = buf[_yybytes_len+1] = YY_END_OF_BUFFER_CHAR;

	b = yy_scan_buffer( buf, n , yyscanner);
	if ( ! b )
		YY_FATAL_ERROR( "bad buffer in yy_scan_bytes()" );

	/* It's okay to grow etc. this buffer, and we should throw it
	 * away when we're done.
	 */
	b->yy_is_our_buffer = 1;

	return b;
}
/* %endif */

#ifndef YY_EXIT_FAILURE
#define YY_EXIT_FAILURE 2
#endif

/* %if-c-only */
static void yynoreturn yy_fatal_error (const char* msg , yyscan_t yyscanner)
{
	struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
	(void)yyg;
	fprintf( stderr, "%s\n", msg );
	exit( YY_EXIT_FAILURE );
}
/* %endif */
/* %if-c++-only */
/* %endif */

/* Redefine yyless() so it works in section 3 code. */

#undef yyless
#define yyless(n) \
	do \
		{ \
		/* Undo effects of setting up yytext. */ \
        int yyless_macro_arg = (n); \
        YY_LESS_LINENO(yyless_macro_arg);\
		yytext[yyleng] = yyg->yy_hold_char; \
		yyg->yy_c_buf_p = yytext + yyless_macro_arg; \
		yyg->yy_hold_char = *yyg->yy_c_buf_p; \
		*yyg->yy_c_buf_p = '\0'; \
		yyleng = yyless_macro_arg; \
		} \
	while ( 0 )

/* Accessor  methods (get/set functions) to struct members. */

/* %if-c-only */
/* %if-reentrant */

/** Get the user-defined data for this scanner.
 * @param yyscanner The scanner object.
 */
YY_EXTRA_TYPE yyget_extra  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    return yyextra;
}

/* %endif */

/** Get the current line number.
 * @param yyscanner The scanner object.
 */
int yyget_lineno  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

        if (! YY_CURRENT_BUFFER)
            return 0;
    
    return yylineno;
}

/** Get the current column number.
 * @param yyscanner The scanner object.
 */
int yyget_column  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

        if (! YY_CURRENT_BUFFER)
            return 0;
    
    return yycolumn;
}

/** Get the input stream.
 * @param yyscanner The scanner object.
 */
FILE *yyget_in  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    return yyin;
}

/** Get the output stream.
 * @param yyscanner The scanner object.
 */
FILE *yyget_out  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    return yyout;
}

/** Get the length of the current token.
 * @param yyscanner The scanner object.
 */
int yyget_leng  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    return yyleng;
}

/** Get the current token.
 * @param yyscanner The scanner object.
 */

char *yyget_text  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    return yytext;
}

/* %if-reentrant */

/** Set the user-defined data. This data is never touched by the scanner.
 * @param user_defined The data to be associated with this scanner.
 * @param yyscanner The scanner object.
 */
void yyset_extra (YY_EXTRA_TYPE  user_defined , yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    yyextra = user_defined ;
}

/* %endif */

/** Set the current line number.
 * @param _line_number line number
 * @param yyscanner The scanner object.
 */
void yyset_lineno (int  _line_number , yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

        /* lineno is only valid if an input buffer exists. */
        if (! YY_CURRENT_BUFFER )
           YY_FATAL_ERROR( "yyset_lineno called with no buffer" );
    
    yylineno = _line_number;
}

/** Set the current column.
 * @param _column_no column number
 * @param yyscanner The scanner object.
 */
void yyset_column (int  _column_no , yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

        /* column is only valid if an input buffer exists. */
        if (! YY_CURRENT_BUFFER )
           YY_FATAL_ERROR( "yyset_column called with no buffer" );
    
    yycolumn = _column_no;
}

/** Set the input stream. This does not discard the current
 * input buffer.
 * @param _in_str A readable stream.
 * @param yyscanner The scanner object.
 * @see yy_switch_to_buffer
 */
void yyset_in (FILE *  _in_str , yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    yyin = _in_str ;
}

void yyset_out (FILE *  _out_str , yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    yyout = _out_str ;
}

int yyget_debug  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    return yy_flex_debug;
}

void yyset_debug (int  _bdebug , yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    yy_flex_debug = _bdebug ;
}

/* %endif */

/* %if-reentrant */
/* Accessor methods for yylval and yylloc */

/* %if-bison-bridge */

YYSTYPE * yyget_lval  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    return yylval;
}

void yyset_lval (YYSTYPE *  yylval_param , yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    yylval = yylval_param;
}

YYLTYPE *yyget_lloc  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    return yylloc;
}
    
void yyset_lloc (YYLTYPE *  yylloc_param , yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    yylloc = yylloc_param;
}
    
/* %endif */

/* User-visible API */

/* yylex_init is special because it creates the scanner itself, so it is
 * the ONLY reentrant function that doesn't take the scanner as the last argument.
 * That's why we explicitly handle the declaration, instead of using our macros.
 */
int yylex_init(yyscan_t* ptr_yy_globals)
{
    if (ptr_yy_globals == NULL){
        errno = EINVAL;
        return 1;
    }

    *ptr_yy_globals = (yyscan_t) yyalloc ( sizeof( struct yyguts_t ), NULL );

    if (*ptr_yy_globals == NULL){
        errno = ENOMEM;
        return 1;
    }

    /* By setting to 0xAA, we expose bugs in yy_init_globals. Leave at 0x00 for releases. */
    memset(*ptr_yy_globals,0x00,sizeof(struct yyguts_t));

    return yy_init_globals ( *ptr_yy_globals );
}

/* yylex_init_extra has the same functionality as yylex_init, but follows the
 * convention of taking the scanner as the last argument. Note however, that
 * this is a *pointer* to a scanner, as it will be allocated by this call (and
 * is the reason, too, why this function also must handle its own declaration).
 * The user defined value in the first argument will be available to yyalloc in
 * the yyextra field.
 */
int yylex_init_extra( YY_EXTRA_TYPE yy_user_defined, yyscan_t* ptr_yy_globals )
{
    struct yyguts_t dummy_yyguts;

    yyset_extra (yy_user_defined, &dummy_yyguts);

    if (ptr_yy_globals == NULL){
        errno = EINVAL;
        return 1;
    }

    *ptr_yy_globals = (yyscan_t) yyalloc ( sizeof( struct yyguts_t ), &dummy_yyguts );

    if (*ptr_yy_globals == NULL){
        errno = ENOMEM;
        return 1;
    }

    /* By setting to 0xAA, we expose bugs in
    yy_init_globals. Leave at 0x00 for releases. */
    memset(*ptr_yy_globals,0x00,sizeof(struct yyguts_t));

    yyset_extra (yy_user_defined, *ptr_yy_globals);

    return yy_init_globals ( *ptr_yy_globals );
}

/* %endif if-c-only */

/* %if-c-only */
static int yy_init_globals (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    /* Initialization is the same as for the non-reentrant scanner.
     * This function is called from yylex_destroy(), so don't allocate here.
     */

    yyg->yy_buffer_stack = NULL;
    yyg->yy_buffer_stack_top = 0;
    yyg->yy_buffer_stack_max = 0;
    yyg->yy_c_buf_p = NULL;
    yyg->yy_init = 0;
    yyg->yy_start = 0;

    yyg->yy_start_stack_ptr = 0;
    yyg->yy_start_stack_depth = 0;
    yyg->yy_start_stack =  NULL;

/* Defined in main.c */
#ifdef YY_STDINIT
    yyin = stdin;
    yyout = stdout;
#else
    yyin = NULL;
    yyout = NULL;
#endif

    /* For future reference: Set errno on error, since we are called by
     * yylex_init()
     */
    return 0;
}
/* %endif */

/* %if-c-only SNIP! this currently causes conflicts with the c++ scanner */
/* yylex_destroy is for both reentrant and non-reentrant scanners. */
int yylex_destroy  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

    /* Pop the buffer stack, destroying each element. */
	while(YY_CURRENT_BUFFER){
		yy_delete_buffer( YY_CURRENT_BUFFER , yyscanner );
		YY_CURRENT_BUFFER_LVALUE = NULL;
		yypop_buffer_state(yyscanner);
	}

	/* Destroy the stack itself. */
	yyfree(yyg->yy_buffer_stack , yyscanner);
	yyg->yy_buffer_stack = NULL;

    /* Destroy the start condition stack. */
        yyfree( yyg->yy_start_stack , yyscanner );
        yyg->yy_start_stack = NULL;

    /* Reset the globals. This is important in a non-reentrant scanner so the next time
     * yylex() is called, initialization will occur. */
    yy_init_globals( yyscanner);

/* %if-reentrant */
    /* Destroy the main struct (reentrant only). */
    yyfree ( yyscanner , yyscanner );
    yyscanner = NULL;
/* %endif */
    return 0;
}
/* %endif */

/*
 * Internal utility routines.
 */

#ifndef yytext_ptr
static void yy_flex_strncpy (char* s1, const char * s2, int n , yyscan_t yyscanner)
{
	struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
	(void)yyg;

	int i;
	for ( i = 0; i < n; ++i )
		s1[i] = s2[i];
}
#endif

#ifdef YY_NEED_STRLEN
static int yy_flex_strlen (const char * s , yyscan_t yyscanner)
{
	int n;
	for ( n = 0; s[n]; ++n )
		;

	return n;
}
#endif

void *yyalloc (yy_size_t  size , yyscan_t yyscanner)
{
	struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
	(void)yyg;
	return malloc(size);
}

void *yyrealloc  (void * ptr, yy_size_t  size , yyscan_t yyscanner)
{
	struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
	(void)yyg;

	/* The cast to (char *) in the following accommodates both
	 * implementations that use char* generic pointers, and those
	 * that use void* generic pointers.  It works with the latter
	 * because both ANSI C and C++ allow castless assignment from
	 * any pointer type to void*, and deal with argument conversions
	 * as though doing an assignment.
	 */
	return realloc(ptr, size);
}

void yyfree (void * ptr , yyscan_t yyscanner)
{
	struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
	(void)yyg;
	free( (char *) ptr );	/* see yyrealloc() for (char *) cast */
}

/* %if-tables-serialization definitions */
/* %define-yytables   The name for this specific scanner's tables. */
#define YYTABLES_NAME "yytables"
/* %endif */

/* %ok-for-header */

#line 391 "grecs-lex.l"

/*
 * Echo LEN bytes from TEXT to yyout.
 */
static void
echo(yyscan_t scanner, char const *text, size_t len)
{
	struct parser_control *pctl = yyget_extra(scanner);
	FILE *out = yyget_out(scanner);
	YYLTYPE *lloc = yyget_lloc(scanner);

	if (len == 0) return;
	/* Check if we should print a syncline */
	if (pctl->beginning_of_line && pctl->emit_syncline) {
		fprintf(out, "#line %lu \"%s\"\n",
			(unsigned long)lloc->end.line,
			lloc->end.file);
		pctl->emit_syncline = 0;
	}
	/* Actually echo the text. */
	fwrite(text, len, 1, out);
	/* Determine the beginning of line status. */
	pctl->beginning_of_line = text[len-1] == '\n';
}

static void
comment_warning(yyscan_t scanner)
{
	struct parser_control *pctl = yyget_extra(scanner);
	if (pctl->emit_comment_warning) {
		grecs_warning(yyget_lloc(scanner), 0,
			      _("\"%2.2s\" starts comment here;"
				" use quotes if you want it to be"
				" a part of string, or insert"
				" a whitespace to suppress this warning"),
			      yyget_text(scanner));
		pctl->emit_comment_warning = 0;
	}
}



/*
 * Source inclusion support.
 */
static int
isglob(const char *s)
{
	return s[strcspn(s, "*?[")] != 0;
}

static int
parse_include(yyscan_t scanner, char *text)
{
	struct parser_control *pctl = yyget_extra(scanner);
	char *p;
	int rc = 1;
	int once;

	p = strchr(text, '#');
	p += strspn(p, " \t") + sizeof("include");
	if (strncmp(p, "_once", 5) == 0) {
		once = 1;
		p += 5;
	} else
		once = 0;
	p += strspn(p, " \t");

	if (!*p) {
		grecs_error(yyget_lloc(scanner), 0,
			    _("invalid include statement"));
		return 1;
	} else {
		size_t len;
		int allow_cwd;

		len = strlen(p);

		if (p[0] == '<' && p[len - 1] == '>') {
			allow_cwd = 0;
			p[len - 1] = 0;
			p++;
		} else if (p[0] == '"' && p[len - 1] == '"') {
			allow_cwd = 1;
			p[len - 1] = 0;
			p++;
		} else {
			grecs_error(yyget_lloc(scanner), 0,
				    _("invalid include statement"));
			return 1;
		}

		if (isglob(p)) {
			switch (glob(p, 0, NULL, &pctl->include_glob)) {
			case 0:
				pctl->include_pos = 0;
				pctl->include_once = once;
				break;
			case GLOB_NOSPACE:
				grecs_alloc_die();
			case GLOB_NOMATCH:
				break;
			default:
				grecs_error(yyget_lloc(scanner), 0,
					    _("read error"));
			}
			p = NULL;
		} else if (p[0] != '/') {
			char *q = p;
			p = grecs_find_include_file(q, allow_cwd);
			if (!p)
				grecs_error(yyget_lloc(scanner), 0,
					    _("%s: No such file or directory"),
					    q);
		}
	}

	if (p)
		rc = push_source(scanner, p, once);
	else if (pctl->include_pos < pctl->include_glob.gl_pathc)
		rc = push_source(scanner,
			      pctl->include_glob.gl_pathv[pctl->include_pos++],
				 once);
	return rc;
}

/*
 * Include search path.
 */
static struct grecs_list *grecs_usr_include_path;
static struct grecs_list *grecs_std_include_path;

size_t
grecs_include_path_count(int flag)
{
    size_t count = 0;
    if (flag & GRECS_STD_INCLUDE)
	count += grecs_list_size(grecs_std_include_path);
    if (flag & GRECS_USR_INCLUDE)
	count += grecs_list_size(grecs_usr_include_path);
    return count;
}

static int
foreach_dir(struct grecs_list *list, int flag,
	    int (*fun)(int, const char *, void *), void *data)
{
    int rc = 0;
    struct grecs_list_entry *ep;

    for (ep = list->head; rc == 0 && ep; ep = ep->next)
	rc = fun(flag, ep->data, data);
    return rc;
}

int
grecs_foreach_include_dir(int flag, int (*fun)(int, const char *, void *),
			  void *data)
{
    int rc = 0;

    if (flag & GRECS_STD_INCLUDE)
	rc = foreach_dir(grecs_std_include_path, GRECS_STD_INCLUDE, fun, data);
    if (rc == 0 && (flag & GRECS_USR_INCLUDE))
	rc = foreach_dir(grecs_usr_include_path, GRECS_USR_INCLUDE, fun, data);
    return rc;
}

void
grecs_include_path_clear()
{
	if (grecs_usr_include_path)
		grecs_list_clear(grecs_usr_include_path);
	if (grecs_std_include_path)
		grecs_list_clear(grecs_std_include_path);
}

static void
incl_free(void *data)
{
	grecs_free(data);
}

void
grecs_include_path_setup_v(char **dirs)
{
	if (!grecs_usr_include_path) {
		grecs_usr_include_path = grecs_list_create();
		grecs_usr_include_path->free_entry = incl_free;
	}
	grecs_std_include_path = grecs_list_create();
	grecs_std_include_path->free_entry = incl_free;
	if (dirs) {
		int i;
		for (i = 0; dirs[i]; i++)
			/* FIXME: Element never freed */
			grecs_list_append(grecs_std_include_path,
					  grecs_strdup(dirs[i]));
	}
}

void
grecs_include_path_setup(const char *dir, ...)
{
	const char *p;
	char **argv = NULL;
	size_t argc = 0;
	size_t argi = 0;
	va_list ap;

	va_start(ap, dir);
	p = dir;
	while (1) {
		if (argi == argc) {
			if (argc == 0)
				argc = 16;
			else
				argc += 16;
			argv = grecs_realloc(argv, argc * sizeof(argv[0]));
		}
		argv[argi++] = (char*) p;
		if (!p)
			break;
		p = va_arg(ap, const char*);
	}
	grecs_include_path_setup_v(argv);
	grecs_free(argv);
	va_end(ap);
}

void
grecs_preproc_add_include_dir(char *dir)
{
	if (!grecs_usr_include_path) {
		grecs_usr_include_path = grecs_list_create();
		grecs_usr_include_path->free_entry = incl_free;
	}
	grecs_list_append(grecs_usr_include_path, grecs_strdup(dir));
}

struct file_data {
	const char *name;
	size_t namelen;
	char *buf;
	size_t buflen;
	int found;
};

static int
pp_list_find(struct grecs_list *list, struct file_data *dptr)
{
	struct grecs_list_entry *ep;

	if (!list)
		return 0;
	for (ep = list->head; !dptr->found && ep; ep = ep->next) {
		const char *dir = ep->data;
		size_t size = strlen (dir) + 1 + dptr->namelen + 1;
		if (size > dptr->buflen) {
			dptr->buflen = size;
			dptr->buf = grecs_realloc(dptr->buf, dptr->buflen);
		}
		strcpy(dptr->buf, dir);
		strcat(dptr->buf, "/");
		strcat(dptr->buf, dptr->name);
		dptr->found = access(dptr->buf, F_OK) == 0;
	}
	return dptr->found;
}

char *
grecs_find_include_file(const char *name, int allow_cwd)
{
	static char *cwd = ".";
	struct file_data fd;

	fd.name = name;
	fd.namelen = strlen(name);
	fd.buf = NULL;
	fd.buflen = 0;
	fd.found = 0;

	if (!grecs_usr_include_path)
		grecs_include_path_setup(NULL);
	if (allow_cwd) {
		grecs_list_append(grecs_usr_include_path, cwd);
		pp_list_find(grecs_usr_include_path, &fd);
		grecs_list_remove_tail(grecs_usr_include_path);
	} else
		pp_list_find(grecs_usr_include_path, &fd);

	if (!fd.found) {
		pp_list_find(grecs_std_include_path, &fd);
		if (!fd.found)
			return NULL;
	}
	return fd.buf;
}

/*
 * Input context support.
 */

struct buffer_ctx {
	struct buffer_ctx *prev;	/* Pointer to previous context */
	struct grecs_locus locus;
	struct grecs_locus_point point;	/* Current input location */
	size_t namelen;		/* Length of the file name */
	struct input_file_ident id;
	FILE *infile;
	YY_BUFFER_STATE state;
};


static void
sigchld_default(struct parser_control *pctl)
{
	if (!pctl->sigchld_reset) {
		struct sigaction act;
		act.sa_flags = 0;
		act.sa_handler = SIG_DFL;
		sigemptyset(&act.sa_mask);
		sigaction(SIGCHLD, &act, &pctl->oldsigchld);
		pctl->sigchld_reset = 1;
	}
}

static void
sigchld_restore(struct parser_control *pctl)
{
	if (pctl->sigchld_reset) {
		sigaction(SIGCHLD, &pctl->oldsigchld, NULL);
		pctl->sigchld_reset = 0;
	}
}

#define STAT_ID_EQ(st,id) ((id).i_node == (st).st_ino       \
			    && (id).device == (st).st_dev)

static struct buffer_ctx *
ctx_lookup(yyscan_t scanner, struct stat *st)
{
	struct parser_control *pctl = yyget_extra(scanner);
	struct buffer_ctx *ctx;

	if (!pctl->context_stack)
		return NULL;

	for (ctx = pctl->context_stack; ctx; ctx = ctx->prev)
		if (STAT_ID_EQ(*st, ctx->id))
			break;
	return ctx;
}

static inline YY_BUFFER_STATE
yyget_current_buffer(yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    return YY_CURRENT_BUFFER;
}

static void
ctx_abort(yyscan_t scanner)
{
	struct parser_control *pctl = yyget_extra(scanner);

	/* Delete current input buffer */
	yy_delete_buffer(yyget_current_buffer(scanner), scanner);

	/* Clean-up context stack */
	while (pctl->context_stack) {
		struct buffer_ctx *prev = pctl->context_stack->prev;
		if (pctl->context_stack->id.pid > 0)
			kill(pctl->context_stack->id.pid, SIGKILL);
		fclose(pctl->context_stack->infile);
		grecs_free(pctl->context_stack);
		pctl->context_stack = prev;
	}
}

/* Calculate the hash of a struct input_file_ident.  */
static unsigned
incl_hasher(void *data, unsigned long n_buckets)
{
	const struct input_file_ident *id = data;
	return (id->i_node + id->device) % n_buckets;
}

/* Compare two input_file_idents for equality.  */
static int
incl_compare(void const *data1, void const *data2)
{
	const struct input_file_ident *id1 = data1;
	const struct input_file_ident *id2 = data2;
	return !(id1->device == id2->device && id1->i_node == id2->i_node);
}

static int
incl_copy(void *dst, void *src)
{
	memcpy(dst, src, sizeof(struct input_file_ident));
	return 0;
}

static int
source_lookup(yyscan_t scanner, struct stat *st)
{
	struct parser_control *pctl = yyget_extra(scanner);
	struct input_file_ident key;
	int install = 1;

	if (!pctl->incl_sources) {
		pctl->incl_sources = grecs_symtab_create(
			sizeof(struct input_file_ident),
			incl_hasher,
			incl_compare,
			incl_copy,
			NULL,/*FIXME: alloc*/
			NULL);
		if (!pctl->incl_sources)
			grecs_alloc_die();
	}

	key.i_node = st->st_ino;
	key.device = st->st_dev;
	if (!grecs_symtab_lookup_or_install(pctl->incl_sources, &key,
					    &install))
		grecs_alloc_die();
	return !install;
}

static int
push_source(yyscan_t scanner, const char *name, int once)
{
	struct parser_control *pctl = yyget_extra(scanner);
	YYLTYPE *lloc = yyget_lloc(scanner);
	FILE *fp;
	pid_t pid;
	struct buffer_ctx *ctx;
	struct stat st;
	int rc = stat(name, &st);

	if (pctl->context_stack) {
		if (rc) {
			grecs_error(lloc, errno,
				    _("Cannot stat `%s'"), name);
			return 1;
		}

		if (pctl->point.col &&
		    STAT_ID_EQ(st, pctl->input_file_ident)) {
			grecs_error(lloc, 0, _("Recursive inclusion"));
			return 1;
		}

		if ((ctx = ctx_lookup(scanner, &st))) {
			grecs_error(lloc, 0,
				    _("Recursive inclusion"));
			if (ctx->prev)
				grecs_error(&ctx->prev->locus, 0,
					    _("`%s' already included here"),
					    name);
			else
				grecs_error(lloc, 0,
					    _("`%s' already included at"
					      " top level"),
					    name);
			return 1;
		}
	} else if (rc) {
		grecs_error(NULL, errno, _("Cannot stat `%s'"), name);
		return 1;
	}

	if (once && source_lookup(scanner, &st))
		return -1;

	if (grecs_preprocessor) {
		if (access(name,R_OK)) {
			grecs_error(pctl->context_stack ? lloc : NULL,
				    errno,
				    _("Cannot open `%s'"), name);
			return -1;
		}
		if (!pctl->context_stack) {
			/*
			 * Ensure default handling of SIGCHLD.
			 * It will be reset back to its original state
			 * by pop_source, when it has popped off the initial
			 * context.
			 */
			sigchld_default(pctl);
		}
		fflush(yyget_out(scanner));
		fp = grecs_preproc_extrn_start(name, &pid);
	} else {
		fp = fopen(name, "r");
		pid = -1;
	}
	if (!fp) {
		grecs_error(pctl->context_stack ? lloc : NULL,
			    errno,
			    _("Cannot open `%s'"), name);
		if (!pctl->context_stack)
			sigchld_restore(pctl);
		return 1;
	}

	/* Push current context */
	if (yyget_in (scanner)) {
		ctx = grecs_malloc(sizeof(*ctx));
		ctx->locus = *lloc;
		ctx->point = pctl->point;
		ctx->id = pctl->input_file_ident;
		ctx->infile = yyget_in(scanner);
		ctx->state = yyget_current_buffer(scanner);
		ctx->prev = pctl->context_stack;
		pctl->context_stack = ctx;
	}

	yyset_in(fp, scanner);
	yy_switch_to_buffer(yy_create_buffer(fp, YY_BUF_SIZE, scanner),
			    scanner);

	pctl->point.file = grecs_install_text(name);
	pctl->point.line = 1;
	pctl->point.col = 0;

	pctl->input_file_ident.i_node = st.st_ino;
	pctl->input_file_ident.device = st.st_dev;
	pctl->input_file_ident.pid = pid;

	pctl->beginning_of_line = 1;

	if (yyget_debug(scanner))
	  fprintf (stderr, "Processing file `%s'\n", name);

	if (pctl->preproc_mode)
		pctl->emit_syncline = 1;

	return 0;
}

static int
pop_source(yyscan_t scanner)
{
	struct parser_control *pctl = yyget_extra(scanner);
	FILE *in = yyget_in(scanner);
	struct buffer_ctx *ctx;

	fclose(in);
	if (pctl->input_file_ident.pid > 0) {
		if (grecs_preproc_extrn_shutdown(pctl->input_file_ident.pid)) {
			/* Abort further input */
			ctx_abort(scanner);

			/* Suppress further errors */
			pctl->error_suppress = 1;

			/* Indicate there is no more input */
			return 1;
		}
	}

	if (!pctl->context_stack) {
		if (yyget_debug(scanner))
			fprintf(stderr, "End of input\n");
		/* Reset SIGCHLD to its original configuration */
		sigchld_restore(pctl);
		yyset_in(NULL, scanner);
		return 1;
	}

	/* Restore previous context */
	yyset_in(pctl->context_stack->infile, scanner);
	pctl->point = pctl->context_stack->point;
	pctl->input_file_ident = pctl->context_stack->id;

	yy_delete_buffer(yyget_current_buffer(scanner), scanner);
	yy_switch_to_buffer(pctl->context_stack->state, scanner);

	ctx = pctl->context_stack->prev;
	grecs_free(pctl->context_stack);
	pctl->context_stack = ctx;

	while (pctl->include_pos < pctl->include_glob.gl_pathc) {
		if (push_source(scanner,
			      pctl->include_glob.gl_pathv[pctl->include_pos++],
			      pctl->include_once) == 0)
			return 0;
	}

	if (pctl->include_glob.gl_pathc) {
		globfree(&pctl->include_glob);
		pctl->include_pos = pctl->include_glob.gl_pathc = 0;
	}

	if (yyget_debug(scanner))
		fprintf(stderr, "Resuming file `%s' at line %lu\n",
			pctl->point.file,
			(unsigned long) pctl->point.line);

	if (pctl->preproc_mode) {
		input(scanner);
		grecs_locus_point_advance_line(pctl->point);
		pctl->emit_syncline = 1;
	}

	return 0;
}

/*
 * Internal lexer functions.
 */
int
yywrap(yyscan_t scanner)
{
	return pop_source(scanner);
}

int
grecs_lex_begin(yyscan_t scanner, const char *name, int trace)
{
	struct parser_control *pctl = yyget_extra(scanner);
	yyset_debug(trace, scanner);
	pctl->acc = grecs_line_acc_create();
	return push_source(scanner, name, 1);
}

void
grecs_lex_end(struct parser_control *pctl)
{
	grecs_symtab_clear(pctl->incl_sources);
	free(pctl->multiline_delimiter);
	grecs_line_acc_free(pctl->acc);
}

/*
 * Here-document functions.
 */
static int
isemptystr(char const *text, int off)
{
	for (; text[off] && isspace(text[off]); off++)
		;
	return text[off] == ';' || text[off] == 0;
}

char *
multiline_strip_tabs(char *text)
{
	if (char_to_strip)
		for (; *text && char_to_strip(*text); text++)
			;
	return text;
}

static void
multiline_add(yyscan_t scanner, char *s)
{
	struct parser_control *pctl = yyget_extra(scanner);

	if (pctl->multiline_unescape) {
		for (; *s; s++)	{
			if (*s == '\\') {
				grecs_line_acc_grow_char_unescape(pctl->acc,
								  s[1]);
				++s;
			} else
				grecs_line_acc_grow_char(pctl->acc, *s);
		}
	} else
		grecs_line_add(pctl->acc, s, strlen(s));
}

static int
is_tab(char c)
{
	return c == '\t';
}

static int
is_ws(char c)
{
	return c == '\t' || c == ' ';
}

void
multiline_begin(yyscan_t scanner, char *p)
{
	struct parser_control *pctl = yyget_extra(scanner);

	if (*p == '-') {
		if (*++p == ' ') {
			char_to_strip = is_ws;
			p++;
		} else
			char_to_strip = is_tab;
	} else
		char_to_strip = NULL;
	if (*p == '\\') {
		p++;
		pctl->multiline_unescape = 0;
		pctl->multiline_delimiter_len = strcspn(p, " \t");
	} else if (*p == '"') {
		char *q;

		p++;
		pctl->multiline_unescape = 0;
		q = strchr(p, '"');
		pctl->multiline_delimiter_len = q - p;
	} else {
		pctl->multiline_delimiter_len = strcspn(p, " \t");
		pctl->multiline_unescape = 1;
	}

	/* Remove trailing newline */
	pctl->multiline_delimiter_len--;
	pctl->multiline_delimiter =
		grecs_malloc(pctl->multiline_delimiter_len + 1);
	memcpy(pctl->multiline_delimiter, p, pctl->multiline_delimiter_len);
	pctl->multiline_delimiter[pctl->multiline_delimiter_len] = 0;
	grecs_line_begin(&pctl->acc);

	/* Update locus */
	grecs_locus_point_advance_line(pctl->point);
	if (grecs_parser_options & GRECS_OPTION_ADJUST_STRING_LOCATIONS) {
		YYLTYPE *lloc = yyget_lloc(scanner);
		lloc->beg = pctl->point;
		lloc->beg.col++;
	}
}

/*
 * Auxiliary lexer functions.
 */
static int
string_or_ident(char const *p)
{
	if (isalpha(*p) || *p == '_') {
		while (*++p) {
			if (!(isalnum(*p) || *p == '_' || *p == '-'))
				return STRING;
		}
		return IDENT;
	}
	return STRING;
}

static void
qstring_locus_fixup(yyscan_t scanner)
{
	YYLTYPE *lloc = yyget_lloc(scanner);
	if (grecs_parser_options & GRECS_OPTION_ADJUST_STRING_LOCATIONS) {
		lloc->beg.col++;
		lloc->end.col--;
	}
}

grecs_value_t *
grecs_value_ptr_from_static(grecs_value_t *input)
{
	grecs_value_t *ptr = grecs_malloc(sizeof(*ptr));
	*ptr = *input;
	return ptr;
}

/*
 * Syncline directive parsers.
 */

static int
assign_locus(struct grecs_locus_point *ploc, char *name, char *line)
{
	char *p;

	if (name) {
		ploc->file = grecs_install_text(name);
	}
	ploc->line = strtoul(line, &p, 10);
	ploc->col = 0;
	return *p != 0;
}

void
grecs_parse_line_directive(char *text, grecs_locus_t *ploc,
			   struct grecs_locus_point *ppoint)
{
	int rc = 1;
	struct wordsplit ws;

	if (wordsplit(text, &ws, WRDSF_DEFFLAGS))
		grecs_error(ploc, 0, _("cannot parse #line line: %s"),
			    wordsplit_strerror(&ws));
	else if (ws.ws_wordc == 2)
		rc = assign_locus(ppoint, NULL, ws.ws_wordv[1]);
	else if (ws.ws_wordc >= 3)
		rc = assign_locus(ppoint, ws.ws_wordv[2], ws.ws_wordv[1]);
	else
		grecs_error(ploc, 0, _("invalid #line statement"));

	if (rc)
		grecs_error(ploc, 0, _("malformed #line statement"));
	wordsplit_free(&ws);
}

void
grecs_parse_line_directive_cpp(char *text, grecs_locus_t *ploc,
			       struct grecs_locus_point *ppoint)
{
	struct wordsplit ws;

	if (wordsplit(text, &ws, WRDSF_DEFFLAGS))
		grecs_error(ploc, 0, _("cannot parse #line line: %s"),
			    wordsplit_strerror(&ws));
	else if (ws.ws_wordc < 3)
		grecs_error(ploc, 0, _("invalid #line statement"));
	else if (assign_locus(ppoint, ws.ws_wordv[2], ws.ws_wordv[1]))
		grecs_error(ploc, 0, _("malformed #line statement"));
	wordsplit_free(&ws);
}

/*
 * External preprocessor support.
 */
const char *grecs_preprocessor = NULL; /* Preprocessor command */
int grecs_log_to_stderr = 1;           /* Log to stderr or to syslog? */
void (*grecs_log_setup_hook) () = NULL;/* Log subsystem setup routine. */

static int
preproc_file(const char *file)
{
	size_t size = 0;
	char *cmd = NULL;
	char *setup_file;
	char *argv[4];
	int rc;

	setup_file = grecs_find_include_file("pp-setup", 0);
	if (setup_file)
		rc = grecs_asprintf(&cmd, &size,
				    "%s \"%s\" \"%s\"",
				    grecs_preprocessor, setup_file, file);
	else
		rc = grecs_asprintf(&cmd, &size,
				    "%s \"%s\"", grecs_preprocessor, file);
	if (rc)
		grecs_alloc_die();
	if ((argv[0] = getenv("SHELL")) == NULL)
		argv[0] = "/bin/sh";
	argv[1] = "-c";
	argv[2] = cmd;
	argv[3] = NULL;
	execvp(argv[0], argv);
	return -1;
}

FILE *
grecs_preproc_extrn_start(const char *file_name, pid_t *ppid)
{
	int pout[2];
	pid_t pid;
	int i;
	FILE *fp = NULL;
	int rc;

	if (pipe(pout)) {
		grecs_error(NULL, errno, "pipe");
		return NULL;
	}
	switch (pid = fork()) {
		/* The child branch.  */
	case 0:
		if (pout[1] != 1) {
			if (dup2(pout[1], 1) == -1) {
				grecs_error(NULL, errno, "dup2");
				exit(127);
			}
		}

		if (!grecs_log_to_stderr) {
			int p[2];
			char *buf = NULL;
			size_t size = 0;
			FILE *fp;

			signal(SIGCHLD, SIG_DFL);
			if (pipe(p)) {
				grecs_error(NULL, errno, "pipe");
				exit(127);
			}
			switch (pid = fork()) {
				/* Grandchild */
			case 0:
				if (p[1] != 2 && dup2(p[1], 2) == -1) {
					grecs_error(NULL, errno, "dup2");
					exit(127);
				}
				close(p[0]);

				rc = preproc_file(file_name);
				break;

			case -1:
				/*  Fork failed */
				if (grecs_log_setup_hook)
					grecs_log_setup_hook();
				grecs_error(NULL, errno, _("Cannot run `%s'"),
					    grecs_preprocessor);
				exit(127);

			default:
				/* Sub-master */
				close (p[1]);
				fp = fdopen(p[0], "r");
				if (grecs_log_setup_hook)
					grecs_log_setup_hook();
				while (grecs_getline(&buf, &size, fp) > 0) {
					size_t n = strlen(buf);
					if (buf[n-1] == '\n')
						buf[n-1] = 0;
					grecs_error(NULL, 0, "%s", buf);
				}
				wait(&i);
				if (WIFEXITED(i))
					rc = WEXITSTATUS(i);
				else
					rc = 127;
			}
		} else {
			rc = preproc_file(file_name);
		}
		if (rc == -1)
			exit(127);
		exit(rc);

	case -1:
		/*  Fork failed */
		grecs_error(NULL, errno, _("Cannot run `%s'"),
			    grecs_preprocessor);
		break;

	default:
		close(pout[1]);
		fp = fdopen(pout[0], "r");
		break;
	}
	*ppid = pid;
	return fp;
}

int
grecs_preproc_extrn_shutdown(pid_t pid)
{
	int status;

	if (waitpid(pid, &status, 0) != pid) {
		grecs_error(NULL, errno, "waitpid(%d)", pid);
	} else if (WIFEXITED(status)) {
		if (WEXITSTATUS(status)) {
			grecs_error(NULL, 0, _("%s exited with status %d"),
				    grecs_preprocessor, WEXITSTATUS(status));
			return -1;
		}
	} else if (WIFSIGNALED(status)) {
		grecs_error(NULL, 0, _("%s terminated on signal %d"),
			    grecs_preprocessor, WTERMSIG(status));
		return -1;
	} else {
		grecs_error(NULL, 0, _("%s terminated abnormally (%d)"),
			    grecs_preprocessor, status);
		return -1;
	}
	return 0;
}

int
grecs_preprocess(char const *name, int trace)
{
	void *scanner;
	struct parser_control ctl = { NULL };
	int t;
	YYSTYPE lval;
	YYLTYPE lloc;

	memset(&lval, 0, sizeof (lval));
	memset(&lloc, 0, sizeof (lloc));

	grecs_grecs_lex_init_extra(&ctl, &scanner);
	yyget_extra(scanner)->preproc_mode = 1;
	if (grecs_lex_begin(scanner, name, trace)) {
		grecs_grecs_lex_destroy(scanner);
		return -1;
	}

	while ((t = yylex(&lval, &lloc, scanner)) != 0) {
		if (trace) {
			char *p;

			YY_LOCATION_PRINT(stderr, lloc);
			fprintf(stderr, ": %d: ", t);
			switch (t) {
			case STRING:
			case QSTRING:
			case MSTRING:
				fputc('"', stderr);
				for (p = lval.string; *p; p++) {
					if (*p == '\\' || *p == '"')
						fputc('\\', stderr);
					fputc(*p, stderr);
				}
				fputc('"', stderr);
				break;

			case IDENT:
				fprintf(stderr, "%s", lval.string);
				break;

			default:
				fputc(t, stderr);
			}
			fputc('\n', stderr);
		}
	}
	grecs_grecs_lex_destroy(scanner);
	grecs_lex_end(&ctl);
	return 0;
}

/*
 * Additional function.
 *
 * Not used by the lexer, but retained for backward compatibility.
 */
ssize_t
grecs_getline(char **pbuf, size_t *psize, FILE *fp)
{
	char *buf = *pbuf;
	size_t size = *psize;
	ssize_t off = 0;

	if (!buf) {
		size = 1;
		buf = grecs_malloc(size);
	}

	do {
		if (off == size - 1) {
			size_t nsize = 2 * size;
			if (nsize < size)
				grecs_alloc_die();
			buf = grecs_realloc(buf, nsize);
			size = nsize;
		}
		if (!fgets(buf + off, size - off, fp)) {
			if (off == 0)
				off = -1;
			break;
		}
		off += strlen(buf + off);
	} while (buf[off - 1] != '\n');

	*pbuf = buf;
	*psize = size;
	return off;
}

